@@ -1,4 +1,4 @@
-/* $Id: cpl_error.c,v 1.72 2011/01/26 12:14:54 llundin Exp $
+/* $Id: cpl_error.c,v 1.73 2012/04/05 14:34:20 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/26 12:14:54 $
- * $Revision: 1.72 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/05 14:34:20 $
+ * $Revision: 1.73 $
+ * $Name: cpl-6_1_1 $
  */
 
 
@@ -132,8 +132,11 @@ static cpl_error * cpl_error_fill(const char *, cpl_error_code, const char *,
 static cpl_error_code
 cpl_error_set_message_macro_(const char *, cpl_error_code,
                              const char *, unsigned,
-                             const char *, va_list) CPL_ATTR_PRINTF(5,0);
-
+                             const char *, va_list) CPL_ATTR_PRINTF(5,0)
+#ifdef CPL_HAVE_ATTR_NONNULL
+    __attribute__((nonnull(3)))
+#endif
+    ;
 
 /*-----------------------------------------------------------------------------
                                   Definitions of functions 
@@ -166,6 +169,7 @@ void cpl_error_reset(void)
 }
 
 /*
+ * @internal
  * @brief
  *   Set CPL error code, function name, source file & line number where it
  *   occurred along with a text message
@@ -894,6 +898,41 @@ cpl_error_code cpl_error_set_message_macro_(const char * function,
                                             const char * file, unsigned line,
                                             const char * text, va_list arglist)
 {
+    /* Check copied from cpl_error_set_message_one_macro() */
+    char * message = (text != NULL && text[0] != '\0' &&
+                      (text[0] != ' ' || text[1] != '\0'))
+        ? cpl_vsprintf(text, arglist) : NULL;
+    const char * usemsg = message ? message : text;
+
+    const cpl_error_code lcode = cpl_error_set_message_one_macro(function, code,
+                                                                 file, line,
+                                                                 usemsg);
+    cpl_free((void*)message);
+
+    return lcode;
+}
+
+
+/*
+ * @internal
+ * @brief
+ *   Set CPL error code, function name, source file & line number where it
+ *   occurred along with a text message and a variable argument list
+ * @param function Character string with function name (cpl_func)
+ * @param code     Error code
+ * @param file     Character string with source file name (__FILE__)
+ * @param line     Integer with line number (__LINE__)
+ * @param text     Text to append to error message
+ * @return The CPL error code, or CPL_ERROR_UNSPECIFIED if the code is 
+ *         CPL_ERROR_HISTORY_LOST.
+ * @see cpl_error_set_message_macro_()
+ */
+cpl_error_code cpl_error_set_message_one_macro(const char * function,
+                                               cpl_error_code code,
+                                               const char * file,
+                                               unsigned line,
+                                               const char * text)
+{
 
     const cpl_error_code lcode = code == CPL_ERROR_HISTORY_LOST
         ? CPL_ERROR_UNSPECIFIED : code;
@@ -914,22 +953,14 @@ cpl_error_code cpl_error_set_message_macro_(const char * function,
                that do not set a message, call with a single space causes that
                user message to be ignored. */
 
-            char message[CPL_ERROR_MAX_MESSAGE_LENGTH];
-            int nlen;
+            /* Concatenate the standard message and the user supplied message */
 
-            nlen = (int)cx_vsnprintf((cxchar *)message,
-                                     (cxsize)CPL_ERROR_MAX_MESSAGE_LENGTH,
-                                     (const cxchar *)text, arglist);
-            if (nlen > 0) {
-                /* Concatenate the standard message and the user supplied message */
-
-                cx_assert( error != NULL );
-                (void)cx_snprintf((cxchar *)error->msg,
-                                  (cxsize)CPL_ERROR_MAX_MESSAGE_LENGTH,
-                                  (const cxchar *)"%s: %s",
-                                  cpl_error_get_message_default(lcode),
-                                  message);
-            }
+            cx_assert( error != NULL );
+            (void)cx_snprintf((cxchar *)error->msg,
+                              (cxsize)CPL_ERROR_MAX_MESSAGE_LENGTH,
+                              (const cxchar *)"%s: %s",
+                              cpl_error_get_message_default(lcode),
+                              text);
         }
     }
 
