@@ -1,4 +1,4 @@
-/* $Id: cpl_error.h,v 1.79 2010/11/12 14:28:01 llundin Exp $
+/* $Id: cpl_error.h,v 1.81 2012/05/02 15:00:08 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/12 14:28:01 $
- * $Revision: 1.79 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/02 15:00:08 $
+ * $Revision: 1.81 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifndef CPL_ERROR_H
@@ -61,6 +61,7 @@ CPL_BEGIN_DECLS
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_error
+  @hideinitializer
   @brief    Flag to indicate support for variadic macros
   @see cpl_error_set_message()
   @note Unless already defined, tries to detect whether variadic macros are
@@ -88,6 +89,7 @@ CPL_BEGIN_DECLS
 /*----------------------------------------------------------------------------*/
 /**
  * @ingroup cpl_error
+   @hideinitializer
    @brief   Generic error handling macro
    @param   CONDITION    The condition to check
    @param   CODE         The CPL error code to set if @em CONDTION is non-zero
@@ -191,10 +193,10 @@ CPL_BEGIN_DECLS
     do if (!(CONDITION)) {                                                     \
         /* Evaluate exactly once using a name-space protected variable name */ \
         const cpl_error_code cpl_ensure_error = (CODE);                        \
-        (void)cpl_error_set_message_macro(cpl_func, cpl_ensure_error           \
-                                          ? cpl_ensure_error :                 \
-                                          CPL_ERROR_UNSPECIFIED,               \
-                                           __FILE__, __LINE__, MSG);           \
+        (void)cpl_error_set_message_one_macro(cpl_func, cpl_ensure_error       \
+                                              ? cpl_ensure_error :             \
+                                              CPL_ERROR_UNSPECIFIED,           \
+                                              __FILE__, __LINE__, MSG);        \
         ACTION;                                                                \
     } while (0)
 #endif
@@ -203,6 +205,7 @@ CPL_BEGIN_DECLS
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_error
+  @hideinitializer
   @brief    Set an error code and return iff a boolean expression is false
   @param    BOOL      The boolean to evaluate
   @param    ERRCODE   The error code to set
@@ -222,6 +225,7 @@ CPL_BEGIN_DECLS
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_error
+  @hideinitializer
   @brief    Set an error code and return it iff a boolean expression is false
   @param    BOOL      The boolean to evaluate
   @param    ERRCODE   The error code to set and return
@@ -235,6 +239,7 @@ CPL_BEGIN_DECLS
 
 /**
  * @ingroup cpl_error
+ * @hideinitializer
  * @brief
  *   Set CPL error code, function name, source file and line number where 
  *   an error occurred.
@@ -256,6 +261,7 @@ CPL_BEGIN_DECLS
 
 /**
  * @ingroup cpl_error
+ * @hideinitializer
  * @brief Propagate a CPL-error to the current location.
  *
  * @param function  Character string with function name, cpl_func
@@ -283,6 +289,7 @@ CPL_BEGIN_DECLS
 
 /**
  * @ingroup cpl_error
+ * @hideinitializer
  * @brief
  *   Set CPL error code, function name, source file and line number where 
  *   an error occurred along with a text message
@@ -295,12 +302,19 @@ CPL_BEGIN_DECLS
  * @note The message is ignored if it is NULL, empty, or consists of a single
  *        ' ' (space). Otherwise, the user supplied message is appended to
  *       the default message using ': ' (colon+space) as delimiter.
- *       The message may be a printf-like format string supplied with arguments
- *       unless variadic macros are not supported in which case only a
- *       non-format string is accepted.
+ *       If the CPL-based application may use variadic macros, the message
+ *       may be a printf-style format string supplied with matching arguments.
+ *       If variadic macros are not allowed (e.g. when compiling with gcc -ansi)
+ *       only a non-format string is accepted. Please be aware that the format
+ *       string should always be a string literal, otherwise the code may be
+ *       vulnerable to the socalled 'format string exploit'. Sufficiently recent
+ *       versions of gcc supports the option -Wformat-security, which tries to
+ *       warn of this issue. If variadic macros are not supported, a 
+ *       printf-style message can be created prior to the call to
+ *       cpl_error_set_message(), as in the below example.
  *
- * Example of usage:
- *  @code
+ * Examples of usage:
+ * @code
  *
  *    if (image == NULL) {
  *        return cpl_error_set_message(cpl_func, CPL_ERROR_NULL_INPUT,
@@ -309,8 +323,18 @@ CPL_BEGIN_DECLS
  *
  * @endcode
  *
- * Example of usage in a C89 application (e.g. compiling with gcc -ansi):
- *  @code
+ * @code
+ *
+ *    if (error_string != NULL) {
+ *        return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
+ *                                     "%s", error_string);
+ *    }
+ *
+ * @endcode
+ *
+ * Example of usage if and only if variadic macros are unavaiable (e.g. when
+ * compiling with gcc -ansi):
+ * @code
  *
  *    if (image == NULL) {
  *        char * my_txt = cpl_sprintf("Image number %d is NULL", number);
@@ -328,7 +352,7 @@ CPL_BEGIN_DECLS
 #define cpl_error_set_message(function, code, ...)                             \
     cpl_error_set_message_macro(function, code, __FILE__, __LINE__, __VA_ARGS__)
 #else
-#define cpl_error_set_message(function, code, text)                            \
+#define cpl_error_set_message_one(function, code, text)                        \
     cpl_error_set_message_macro(function, code, __FILE__, __LINE__, text)
 #endif
 
@@ -443,6 +467,16 @@ cpl_error_set_message_macro(const char *, cpl_error_code,
     __attribute__((nonnull(3)))
 #endif
      ;
+
+cpl_error_code
+cpl_error_set_message_one_macro(const char *, cpl_error_code,
+                                const char *, unsigned,
+                                const char *)
+#ifdef CPL_HAVE_ATTR_NONNULL
+    __attribute__((nonnull(3)))
+#endif
+    ;
+
 cpl_error_code  cpl_error_get_code(void)     CPL_ATTR_PURE;
 const char     *cpl_error_get_message(void)  CPL_ATTR_PURE;
 const char     *cpl_error_get_message_default(cpl_error_code) CPL_ATTR_CONST;
