@@ -1,4 +1,4 @@
-/* $Id: cpl_fits.c,v 1.20 2012/03/05 13:46:41 llundin Exp $
+/* $Id: cpl_fits.c,v 1.26 2012/05/11 14:30:40 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/05 13:46:41 $
- * $Revision: 1.20 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/11 14:30:40 $
+ * $Revision: 1.26 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,6 +60,113 @@
                             Function codes
  -----------------------------------------------------------------------------*/
 
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief  Set the FITS I/O mode
+  @param  mode The FITS I/O mode: CPL_FITS_STOP_CACHING, CPL_FITS_START_CACHING,
+               CPL_FITS_RESTART_CACHING
+  @return CPL_ERROR_NONE or the relevant #_cpl_error_code_ on error
+  @see cpl_init() to control the FITS I/O mode when CPL is started
+
+   Normally when a FITS file is processed with a CPL call the
+   file is openened and closed during that call. However repeated calls on
+   FITS data with many extensions causes the FITS headers to be parsed many
+   times which can lead to a significant performance penalty.
+   If instead this function is called with CPL_FITS_START_CACHING,
+   CPL will use internal storage to keep the FITS files open between calls and
+   only close them when the FITS I/O is changed (or cpl_end() is called).
+
+   If a CPL function that creates a FITS file is called any previously opened
+   handles to that file are closed. If a CPL function that appends to a FITS
+   file is called any previously opened read-only handles to that file are
+   closed. If a CPL function that reads from a FITS file is called any
+   previously opened read/write-handle to that file is used for the read. Any
+   additional concurrent reads causes the file to also be opened for reading.
+   This means that there is also a performance gain when alternating between
+   appending to and reading from the same file.
+   This optimized I/O mode cannot be used if the CPL application accesses
+   a FITS file without using CPL. An incomplete list of such access is:
+   Calls to rename(), remove(), unlink(), fopen() and access via another
+   process started with for example system() or popen().
+
+   The caching of opened FITS files may be used in a multi-threaded environment
+   to the extent that the underlying FITS library (CFITSIO) supports it.
+   One implication of this
+   is that multiple threads may only call CPL FITS saving functions on the same
+   file using proper synchronization such as the OpenMP 'ordered' construct.
+   CPL makes no attempt to verify that a CPL based application performs thread
+   parallel FITS I/O correctly.
+
+  The mode CPL_FITS_STOP_CACHING causes all cached filed to be closed.
+  Beware that this can cause an I/O error for a file that has otherwise
+  not been accessed for some time.
+
+  Since CPL_FITS_STOP_CACHING closes all cached FITS files, the caller must
+  ensure that this does not interfere with the concurrent use of those same
+  files in another thread.
+
+  The mode CPL_FITS_RESTART_CACHING has the same effect as a call with
+  CPL_FITS_STOP_CACHING followed by a call with CPL_FITS_START_CACHING.
+
+  The modes CPL_FITS_RESTART_CACHING and CPL_FITS_ONE may be combined.
+  This causes all files cached by the calling thread to be closed. The
+  caching remains active (for all threads), so subsequently opened files
+  will be cached.
+
+  Possible #_cpl_error_code_ set in this function:
+  - CPL_ERROR_ILLEGAL_INPUT if the mode is zero
+  - CPL_ERROR_UNSUPPORTED_MODE if the mode is not supported
+  - CPL_ERROR_INVALID_TYPE if mode is 1, e.g. due to a logical or (||)
+      of the allowed options.
+  - CPL_ERROR_BAD_FILE_FORMAT if the I/O caused by CPL_FITS_STOP_CACHING failed
+ */
+/*----------------------------------------------------------------------------*/
+cpl_error_code cpl_fits_set_mode(cpl_fits_mode mode)
+{
+
+    if (mode == CPL_FITS_START_CACHING) {
+
+        cpl_io_fits_init();
+
+    } else if (mode == CPL_FITS_STOP_CACHING) {
+
+        if (cpl_io_fits_end()) return cpl_error_set_where_();
+
+    } else if ((mode & CPL_FITS_RESTART_CACHING) == CPL_FITS_RESTART_CACHING) {
+
+        if (cpl_io_fits_close_tid((mode & CPL_FITS_ONE) ? CPL_IO_FITS_ONE
+                                  : CPL_IO_FITS_ALL))
+            return cpl_error_set_where_();
+
+    } else if (mode == 0) {
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
+    } else if (mode == 1) {
+        return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+    } else {
+        return cpl_error_set_message_(CPL_ERROR_UNSUPPORTED_MODE,
+                                      "mode=%d", (int)mode);
+    }
+
+    return CPL_ERROR_NONE;
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief  Get the FITS I/O mode
+  @return One of: CPL_FITS_STOP_CACHING, CPL_FITS_START_CACHING
+  @see cpl_fits_set_mode()
+
+ */
+/*----------------------------------------------------------------------------*/
+cpl_fits_mode cpl_fits_get_mode(void)
+{
+
+    return cpl_io_fits_is_enabled() ? CPL_FITS_START_CACHING
+        : CPL_FITS_STOP_CACHING;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief  Get the number of extensions contained in a FITS file
