@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic_body.h,v 1.91 2012/03/13 15:03:37 llundin Exp $
+/* $Id: cpl_image_basic_body.h,v 1.99 2012/05/03 12:07:07 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,20 +20,22 @@
 
 #if defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_OPERATE
 case CPL_TYPE_T: {
-    const CPL_TYPE * p2 = (const CPL_TYPE *)image2->pixels;
+    const CPL_TYPE * p2  = (const CPL_TYPE *)image2->pixels;
+    const size_t     nxy = (size_t)(image1->nx * image1->ny);
     size_t           i;
 
-    for (i = 0; i < (size_t)(image1->nx * image1->ny); i++)
+    for (i = 0; i < nxy; i++)
         CPL_OPERATOR(pout[i], p1[i], p2[i]);
     break;
 }
 
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_DIVIDE
 case CPL_TYPE_T: {
-    const CPL_TYPE * p2 = (const CPL_TYPE *)image2->pixels;
+    const CPL_TYPE * p2  = (const CPL_TYPE *)image2->pixels;
+    const size_t     nxy = (size_t)(image1->nx * image1->ny);
     size_t           i;
 
-    for (i = 0; i < (size_t)(image1->nx * image1->ny); i++) {
+    for (i = 0; i < nxy; i++) {
         if (p2[i] != (CPL_TYPE)0) {
             CPL_IMAGE_DIVISION(pout[i], p1[i], p2[i]);
         } else {
@@ -46,20 +48,22 @@ case CPL_TYPE_T: {
 
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_OPERATE_LOCAL
 case CPL_TYPE_T: {
-    const CPL_TYPE * p2 = (const CPL_TYPE *)im2->pixels;
+    const CPL_TYPE * p2  = (const CPL_TYPE *)im2->pixels;
+    const size_t     nxy = (size_t)(im1->nx * im1->ny);
     size_t           i;
 
-    for (i = 0; i < (size_t)(im1->nx * im1->ny); i++) 
+    for (i = 0; i < nxy; i++) 
         CPL_OPERATOR(p1[i], p2[i]);
     break;
 }
 
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_BASIC_DIVIDE_LOCAL
 case CPL_TYPE_T: {
-    const CPL_TYPE * p2 = (const CPL_TYPE * )im2->pixels;
+    const CPL_TYPE * p2  = (const CPL_TYPE * )im2->pixels;
+    const size_t     nxy = (size_t)(im1->nx * im1->ny);
     size_t           i;
 
-    for (i = 0; i < (size_t)(im1->nx * im1->ny); i++) {
+    for (i = 0; i < nxy; i++) {
         if (p2[i] != (CPL_TYPE)0) {
             CPL_IMAGE_DIVISIONASSIGN(p1[i], p2[i]);
         } else {
@@ -124,9 +128,9 @@ case CPL_TYPE_T: {
  -----------------------------------------------------------------------------*/
 
 static cpl_image *
-CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image *,
+CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image *, cpl_size,
                                                cpl_size, cpl_size, cpl_size,
-                                               cpl_size, int);
+                                               int);
 
 static cpl_error_code CPL_TYPE_ADD(cpl_image_power)(cpl_image *, double);
 
@@ -152,11 +156,11 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_logarithm)(cpl_image * self,
     double scale;
     const int myerrno = errno;
     CPL_TYPE * data = cpl_image_get_data(self);
-    const cpl_size n = cpl_image_get_size_x(self) * cpl_image_get_size_y(self);
+    const size_t n = cpl_image_get_size_x(self) * cpl_image_get_size_y(self);
     const cpl_mask * mask = cpl_image_get_bpm_const(self);
     const cpl_binary * cbpm = mask ? cpl_mask_get_data_const(mask) : NULL;
     cpl_binary * bpm = NULL;
-    cpl_size i;
+    size_t i;
 
     cpl_ensure_code(data != NULL, CPL_ERROR_ILLEGAL_INPUT);
     cpl_ensure_code(base >  0.0,  CPL_ERROR_ILLEGAL_INPUT);
@@ -217,11 +221,11 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_exponential)(cpl_image * self,
 
     const int myerrno = errno;
     CPL_TYPE * data = cpl_image_get_data(self);
-    const cpl_size n = cpl_image_get_size_x(self) * cpl_image_get_size_y(self);
+    const size_t n = cpl_image_get_size_x(self) * cpl_image_get_size_y(self);
     const cpl_mask * mask = cpl_image_get_bpm_const(self);
     const cpl_binary * cbpm = mask ? cpl_mask_get_data_const(mask) : NULL;
     cpl_binary * bpm = NULL;
-    cpl_size i;
+    size_t i;
 
     cpl_ensure_code(data != NULL, CPL_ERROR_ILLEGAL_INPUT);
 
@@ -321,11 +325,11 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_power)(cpl_image * self,
 
     const int myerrno = errno;
     CPL_TYPE * data = cpl_image_get_data(self);
-    const cpl_size n = cpl_image_get_size_x(self) * cpl_image_get_size_y(self);
+    const size_t n = cpl_image_get_size_x(self) * cpl_image_get_size_y(self);
     const cpl_mask * mask = cpl_image_get_bpm_const(self);
     const cpl_binary * cbpm = mask ? cpl_mask_get_data_const(mask) : NULL;
     cpl_binary * bpm = NULL;
-    cpl_size i;
+    size_t i;
 
     cpl_ensure_code(data != NULL, CPL_ERROR_ILLEGAL_INPUT);
 
@@ -379,12 +383,12 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_power)(cpl_image * self,
                 }
             }
         }
-    } else if (exponent != ceil(exponent)) {
+    } else if (exponent == 0.5) {
         /* Pixel value must be non-negative */
         for (i = 0; i < n; i++) {
             if (!cbpm || !cbpm[i]) { /* Pixel is not bad (yet) */
                 if ((double)data[i] >= 0.0) {
-                    data[i] = (CPL_TYPE)pow((double)data[i], exponent);
+                    data[i] = (CPL_TYPE)sqrt((double)data[i]);
                 } else {
                     errno = EDOM; /* Use errno to reuse code */
                 }
@@ -401,12 +405,27 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_power)(cpl_image * self,
                 }
             }
         }
-    } else if (exponent == 0.5) {
+#ifdef HAVE_CBRT
+    } else if (exponent == 1.0 / 3.0) {
+        if (cbpm == NULL) {
+            /* No bad pixels */
+            for (i = 0; i < n; i++) {
+                data[i] = (CPL_TYPE)cbrt((double)data[i]);
+            }
+        } else {
+            for (i = 0; i < n; i++) {
+                if (!cbpm[i]) { /* Pixel is not bad */
+                    data[i] = (CPL_TYPE)cbrt((double)data[i]);
+                }
+            }
+        }
+#endif
+    } else if (exponent != ceil(exponent)) {
         /* Pixel value must be non-negative */
         for (i = 0; i < n; i++) {
             if (!cbpm || !cbpm[i]) { /* Pixel is not bad (yet) */
                 if ((double)data[i] >= 0.0) {
-                    data[i] = (CPL_TYPE)sqrt((double)data[i]);
+                    data[i] = (CPL_TYPE)pow((double)data[i], exponent);
                 } else {
                     errno = EDOM; /* Use errno to reuse code */
                 }
@@ -489,9 +508,9 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     const cpl_binary * bpmi = self->bpm ? cpl_mask_get_data(self->bpm) : NULL;
     CPL_TYPE         * po;
     cpl_binary       * bpmo = NULL;
-    const cpl_size     n1x  = 1 + urx - llx;
-    const cpl_size     n1y  = 1 + ury - lly;
-    cpl_size           i, j;
+    const size_t       n1x  = 1 + urx - llx;
+    const size_t       n1y  = 1 + ury - lly;
+    size_t             i, j;
 
 
     cpl_ensure(direction == 0 || direction == 1, CPL_ERROR_ILLEGAL_INPUT, NULL);
@@ -508,14 +527,14 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     if (direction == 0) {
         const double r1y = (double)n1y;
-        cpl_size * nok;
+        size_t * nok;
 
         other = cpl_image_new(n1x, 1, CPL_TYPE_T);
         po = (CPL_TYPE*)other->pixels;
 
         /* To obtain a stride-1 access of pi[], some temporary storage is needed
            - this may be a disadvantage for very small images */
-        nok = (cpl_size*)cpl_calloc((size_t)n1x, sizeof(*nok));
+        nok = (size_t*)cpl_calloc(n1x, sizeof(*nok));
 
         for (j=0; j < n1y; j++) {
             for (i=0; i < n1x; i++) {
@@ -543,7 +562,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     } else if (direction == 1) {
         const double r1x = (double)n1x;
         double       sum;
-        cpl_size     nok;
+        size_t       nok;
 
         other = cpl_image_new(1, n1y, CPL_TYPE_T);
         po = (CPL_TYPE*)other->pixels;
@@ -1017,14 +1036,15 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     
     case CPL_TYPE_T:
     {
-        CPL_TYPE     * pio = (CPL_TYPE*)self->pixels;
-        const cpl_size nxy = self->nx * self->ny;
-        cpl_size       i;
+        CPL_TYPE     * pio     = (CPL_TYPE*)self->pixels;
+        const CPL_TYPE cscalar = (const CPL_TYPE)scalar;
+        const size_t   nxy     = (size_t)(self->nx * self->ny);
+        size_t         i;
 
         assert( self->pixels );    
    
         for (i = 0; i < nxy; i++) {
-            CPL_OPERATOR(pio[i], scalar);
+            CPL_OPERATOR(pio[i], cscalar);
         }
 
 #ifdef CPL_TYPE_IS_FPOINT
@@ -1037,8 +1057,11 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     
     case CPL_TYPE_T:
     {
-        CPL_TYPE * pio = (CPL_TYPE*)image->pixels;
-        for (i=0; i < (image->nx * image->ny); i++)
+        CPL_TYPE   * pio = (CPL_TYPE*)image->pixels;
+        const size_t nxy = (size_t)(image->nx * image->ny);
+        size_t       i;
+
+        for (i=0; i < nxy; i++)
             pio[i] = sqrt(pio[i]);
 #ifdef CPL_TYPE_IS_FPOINT
         cpl_tools_add_flops( image->nx * image->ny );
@@ -1050,9 +1073,11 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     case CPL_TYPE_T:
     {
-        CPL_TYPE * pi;
-        pi = (CPL_TYPE*)image_in->pixels;
-        for (i=0; i<(image_in->nx * image_in->ny); i++) {
+        CPL_TYPE   * pi = (CPL_TYPE*)image_in->pixels;
+        const size_t nxy = (size_t)(image_in->nx * image_in->ny);
+        size_t       i;
+
+        for (i=0; i < nxy; i++) {
             if (pi[i] > hi_cut) pi[i] = (CPL_TYPE)assign_hi_cut;
             else if (pi[i] < lo_cut) pi[i] = (CPL_TYPE)assign_lo_cut;
         }
@@ -1063,8 +1088,11 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
       case CPL_TYPE_T:
       {
-          CPL_TYPE * pio = (CPL_TYPE*)image->pixels;
-          for (i = 0; i < image->nx * image->ny; i++)
+          CPL_TYPE   * pio = (CPL_TYPE*)image->pixels;
+          const size_t nxy = (size_t)(image->nx * image->ny);
+          size_t       i;
+
+          for (i = 0; i < nxy; i++)
               pio[i] = (CPL_TYPE)CPL_MATH_ABS((CPL_MATH_TYPE)pio[i]);
 #ifdef CPL_TYPE_IS_FPOINT
           cpl_tools_add_flops( image->nx * image->ny );
@@ -1077,32 +1105,35 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     
     case CPL_TYPE_T:
     {
+        const size_t     nxy = (size_t)(image_1->nx * image_1->ny);
         const CPL_TYPE * pi1 = (const CPL_TYPE*)image_1->pixels;
-        CPL_TYPE * po;
-        image_out = cpl_image_new(image_1->nx, image_1->ny, CPL_TYPE_T);
-        po = (CPL_TYPE*)image_out->pixels;
+        CPL_TYPE       * po = (CPL_TYPE *)cpl_malloc(nxy * sizeof(*po));
+        size_t           i;
+
         /* Switch on second passed image type */
         switch (image_2->type) {
             case CPL_TYPE_INT:
                 pii2 = (int*)image_2->pixels;
-                for (i=0; i<(image_out->nx * image_out->ny); i++)
+                for (i = 0; i < nxy; i++)
                     po[i] = (CPL_TYPE)(0.5 * (pi1[i] + pii2[i]));
                 break;
             case CPL_TYPE_FLOAT:
                 pfi2 = (float*)image_2->pixels;
-                for (i=0; i<(image_out->nx * image_out->ny); i++)
+                for (i = 0; i < nxy; i++)
                     po[i] = (CPL_TYPE)(0.5 * (pi1[i] + pfi2[i]));
                 break;
             case CPL_TYPE_DOUBLE:
                 pdi2 = (double*)image_2->pixels;
-                for (i=0; i<(image_out->nx * image_out->ny); i++)
+                for (i = 0; i < nxy; i++)
                     po[i] = (CPL_TYPE)(0.5 * (pi1[i] + pdi2[i]));
                 break;
             default:
-                cpl_image_delete(image_out);
+                cpl_free(po);
                 (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
                 return NULL;
         }
+        image_out = cpl_image_wrap_(image_1->nx, image_1->ny, CPL_TYPE_T, po);
+
 #ifdef CPL_TYPE_IS_FPOINT
         cpl_tools_add_flops( 2 * image_out->nx * image_out->ny );
 #endif
@@ -1113,12 +1144,11 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     case CPL_TYPE_T:
     {
-        const cpl_size outlx = urx - llx + 1;
-        const cpl_size outly = ury - lly + 1;
+        const size_t outlx = urx - llx + 1;
+        const size_t outly = ury - lly + 1;
 
         /* Output pixel buffer */
-        void         * po    = cpl_malloc((size_t)outlx * (size_t)outly
-                                          * sizeof(CPL_TYPE));
+        void         * po  = cpl_malloc(outlx * outly * sizeof(CPL_TYPE));
 
         if (cpl_tools_copy_window(po, in->pixels, sizeof(CPL_TYPE),
                                   in->nx, in->ny, llx, lly, urx, ury)) {
@@ -1135,9 +1165,10 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     case CPL_TYPE_T:
     {
-        CPL_TYPE * pi;
-        pi = (CPL_TYPE*)image_in->pixels; 
-        for (i=0; i<image_in->nx; i++) {
+        CPL_TYPE * pi = (CPL_TYPE*)image_in->pixels; 
+        size_t i;
+
+        for (i = 0; i < (size_t)image_in->nx; i++) {
             out_data[i] = (double)pi[i+(pos-1)*image_in->nx];
         }
         break;
@@ -1147,9 +1178,10 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     case CPL_TYPE_T:
     {
-        CPL_TYPE * pi;
-        pi = (CPL_TYPE*)image_in->pixels; 
-        for (i=0; i<image_in->ny; i++) {
+        CPL_TYPE * pi = (CPL_TYPE*)image_in->pixels; 
+        size_t i;
+
+        for (i = 0; i < (size_t)image_in->ny; i++) {
             out_data[i] = (double)pi[pos-1+i*image_in->nx];
         }
         break;
@@ -1160,28 +1192,26 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
     case CPL_TYPE_T:
     {
         /* Number of output pixels */
-        const cpl_size   npix = direction ? self->ny : self->nx;
+        const size_t     npix = direction ? self->ny : self->nx;
 
         /* Max number of input pixels in one median computation */
-        const cpl_size   nmed = (direction ? self->nx : self->ny) - ndrop;
+        const size_t     nmed = (direction ? self->nx : self->ny) - ndrop;
 
         const CPL_TYPE * pi  = (const CPL_TYPE*)self->pixels;
-        CPL_TYPE       * po  = (CPL_TYPE*)cpl_malloc((size_t)npix *
-                                                     sizeof(CPL_TYPE));
-        CPL_TYPE       * med = (CPL_TYPE*)cpl_malloc((size_t)nmed *
-                                                     sizeof(CPL_TYPE));
+        CPL_TYPE       * po  = (CPL_TYPE*)cpl_malloc(npix * sizeof(CPL_TYPE));
+        CPL_TYPE       * med = (CPL_TYPE*)cpl_malloc(nmed * sizeof(CPL_TYPE));
         cpl_binary     * bpm = NULL;
         const cpl_binary * bin = self->bpm
             ? cpl_mask_get_data_const(self->bpm) : NULL;
         cpl_boolean      isok = bin == NULL ? CPL_TRUE : CPL_FALSE;
-        cpl_size         i, j;
+        size_t           i, j;
 
         if (direction == 1) {
             /* Collapsing the image in the x direction */
             pi += drop_ll;
             if (bin == NULL) {
-                for (j = 0; j < self->ny; j++, pi += self->nx) {
-                    cpl_size k = 0;
+                for (j = 0; j < (size_t)(self->ny); j++, pi += self->nx) {
+                    size_t k = 0;
                     for (i = 0; i < nmed; i++) {
                         med[k++] = pi[i];
                     }
@@ -1189,9 +1219,9 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                 }
             } else {
                 bin += drop_ll;
-                for (j = 0; j < self->ny; j++,
+                for (j = 0; j < (size_t)(self->ny); j++,
                          pi += self->nx, bin += self->nx) {
-                    cpl_size k = 0;
+                    size_t k = 0;
                     for (i = 0; i < nmed; i++) {
                         if (!bin[i]) med[k++] = pi[i];
                     }
@@ -1210,8 +1240,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
             /* Collapsing the image in the y direction */
             pi += drop_ll * self->nx;
             if (bin == NULL) {
-                for (i = 0; i < self->nx; i++, pi++) {
-                    cpl_size k = 0;
+                for (i = 0; i < (size_t)(self->nx); i++, pi++) {
+                    size_t k = 0;
                     for (j = 0; j < nmed; j++) {
                         med[k++] = pi[j * self->nx];
                     }
@@ -1219,8 +1249,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                 }
             } else {
                 bin += drop_ll * self->nx;
-                for (i = 0; i < self->nx; i++, pi++, bin++) {
-                    cpl_size k = 0;
+                for (i = 0; i < (size_t)(self->nx); i++, pi++, bin++) {
+                    size_t k = 0;
                     for (j = 0; j < nmed; j++) {
                         if (!bin[j * self->nx]) med[k++] = pi[j * self->nx];
                     }
@@ -1262,7 +1292,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
         switch(rot) {
             case 1: {
                 CPL_TYPE * pi = (CPL_TYPE *)self->pixels;
-                cpl_size   i,j;
+                size_t   i,j;
 
                 if (self->nx == self->ny) {
                     /* If self->nx is even,
@@ -1273,8 +1303,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                     /* The first four pixels to move are the corner ones,
                        followed by their neighbors
                        - the last four pixels to move are the center ones. */
-                    for (j = 0; j < self->ny/2; j++) {
-                        for (i = j; i < self->nx-1-j; i++) {
+                    for (j = 0; j < (size_t)(self->ny/2); j++) {
+                        for (i = j; i < (size_t)(self->nx-1-j); i++) {
                             const CPL_TYPE tmp = pi[i + j * self->nx];
 
                             pi[i + j * self->nx]
@@ -1301,8 +1331,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                     self->nx = tmp->ny;
                     self->ny = tmp->nx;
                     pi += ((self->ny)-1)* (self->nx);
-                    for (j=0; j<(self->nx); j++) {
-                        for (i=0; i<(self->ny); i++) {
+                    for (j=0; j < (size_t)(self->nx); j++) {
+                        for (i = 0; i < (size_t)(self->ny); i++) {
                             *pi = *pt++;
                             pi -= (self->nx);
                         }
@@ -1314,8 +1344,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
             }
             case 2: {
                 CPL_TYPE * pi = (CPL_TYPE *)self->pixels;
-                cpl_size   i;
-                cpl_size   j = self->nx * self->ny - 1;
+                size_t   i;
+                size_t   j = self->nx * self->ny - 1;
 
                 for (i = 0; i < j; i++, j--) {
                     const CPL_TYPE tmp = pi[i];
@@ -1326,11 +1356,11 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
             }
             case 3: {
                 CPL_TYPE * pi = (CPL_TYPE *)self->pixels;
-                cpl_size   i,j;
+                size_t   i,j;
                 if (self->nx == self->ny) {
                     /* See case 1. */
-                    for (j = 0; j < self->ny/2; j++) {
-                        for (i = j; i < self->nx-1-j; i++) {
+                    for (j = 0; j < (size_t)(self->ny/2); j++) {
+                        for (i = j; i < (size_t)(self->nx-1-j); i++) {
                             const CPL_TYPE tmp = pi[i + j * self->nx];
 
                             pi[i + j * self->nx]
@@ -1357,8 +1387,8 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                     self->nx = tmp->ny;
                     self->ny = tmp->nx;
                     pi += (self->nx)-1;
-                    for (j=0; j<(self->nx); j++) {
-                        for (i=0; i<(self->ny); i++) {
+                    for (j=0; j < (size_t)(self->nx); j++) {
+                        for (i = 0; i < (size_t)(self->ny); i++) {
                             *pi = *pt++;
                             pi += (self->nx);
                         }
@@ -1378,10 +1408,10 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
 
     case CPL_TYPE_T:
     {
-        const cpl_size nx = im->nx;
-        const cpl_size ny = im->ny;
-        CPL_TYPE     * pi = (CPL_TYPE *)im->pixels;
-        cpl_size       i, j;
+        const size_t nx = im->nx;
+        const size_t ny = im->ny;
+        CPL_TYPE   * pi = (CPL_TYPE *)im->pixels;
+        size_t       i, j;
 
         switch(angle) {
         case 0: {
@@ -1427,7 +1457,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                 im->nx = ny;
                 im->ny = nx;
                 for (j=0; j<nx; j++) {
-                    for (i=0; i<ny; i++) {
+                    for (i = 0; i<ny; i++) {
                         *pi++ = *pt;
                         pt += nx;
                     }
@@ -1457,7 +1487,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                 im->ny = nx;
                 pt += (nx*ny-1);
                 for (j=0; j<nx; j++) {
-                    for (i=0; i<ny; i++) {
+                    for (i = 0; i<ny; i++) {
                         *pi++ = *pt;
                         pt -= nx;
                     }
@@ -1490,7 +1520,7 @@ CPL_TYPE_ADD(cpl_image_collapse_window_create)(const cpl_image * self,
                         
         /* Move the pixels */
         for (j=0; j<nb_cut; j++) {
-            for (i=0; i<nb_cut; i++) {
+            for (i = 0; i<nb_cut; i++) {
                 tile_x = (new_pos[i+j*nb_cut]-1) % nb_cut;
                 tile_y = (new_pos[i+j*nb_cut]-1) / nb_cut;
                 for (l=0; l<tile_sz_y; l++) {
