@@ -1,4 +1,4 @@
-/* $Id: cpl_image_io_body.h,v 1.56 2012/03/13 09:32:00 llundin Exp $
+/* $Id: cpl_image_io_body.h,v 1.58 2012/03/19 12:42:54 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -21,9 +21,10 @@
 #if defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_RE_IM
 
  if (preal != NULL && pimag != NULL) {
-     size_t i;
+     const size_t nxy = (size_t)(self->nx * self->ny);
+     size_t       i;
 
-     for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+     for (i = 0; i < nxy; i++) {
              preal[i] = CPL_CREAL(pself[i]);
              pimag[i] = CPL_CIMAG(pself[i]);
      }
@@ -34,9 +35,10 @@
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_RE
 
  if (preal != NULL) {
-     size_t i;
+     const size_t nxy = (size_t)(self->nx * self->ny);
+     size_t       i;
 
-     for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+     for (i = 0; i < nxy; i++) {
          preal[i] = CPL_CREAL(pself[i]);
      }
  } else {
@@ -46,9 +48,10 @@
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_IM
 
  if (pimag != NULL) {
-     size_t i;
+     const size_t nxy = (size_t)(self->nx * self->ny);
+     size_t       i;
 
-     for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+     for (i = 0; i < nxy; i++) {
          pimag[i] = CPL_CIMAG(pself[i]);
      }
  } else {
@@ -58,9 +61,10 @@
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_ABS_ARG
 
  if (pabs != NULL && parg != NULL) {
-     size_t i;
+     const size_t nxy = (size_t)(self->nx * self->ny);
+     size_t       i;
 
-     for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+     for (i = 0; i < nxy; i++) {
              pabs[i] = CPL_CABS(pself[i]);
              parg[i] = CPL_CARG(pself[i]);
      }
@@ -71,9 +75,10 @@
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_ABS
 
  if (pabs != NULL) {
-     size_t i;
+     const size_t nxy = (size_t)(self->nx * self->ny);
+     size_t       i;
 
-     for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+     for (i = 0; i < nxy; i++) {
          pabs[i] = CPL_CABS(pself[i]);
      }
  } else {
@@ -83,9 +88,10 @@
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_ARG
 
  if (parg != NULL) {
-     size_t i;
+     const size_t nxy = (size_t)(self->nx * self->ny);
+     size_t       i;
 
-     for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+     for (i = 0; i < nxy; i++) {
          parg[i] = CPL_CARG(pself[i]);
      }
  } else {
@@ -168,11 +174,12 @@
 #elif defined(CPL_OPERATION) && CPL_OPERATION == CPL_IMAGE_IO_SET_BADPIXEL
 
   case CPL_TYPE_T: {
-      CPL_TYPE          *   pi;
-      const CPL_TYPE        acast = (CPL_TYPE) a;
-      int                   i; 
-      pi = (CPL_TYPE*)im->pixels;
-      for (i=0; i<im->nx*im->ny; i++) 
+      CPL_TYPE     * pi = (CPL_TYPE*)im->pixels;
+      const CPL_TYPE acast = (const CPL_TYPE) a;
+      const size_t   nxy = (size_t)(im->nx * im->ny);
+      size_t         i;
+
+      for (i = 0; i < nxy; i++) 
           if (pbpm[i] == CPL_BINARY_1) pi[i] = acast;
       break;
   }
@@ -205,6 +212,7 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_conjugate)(cpl_image *,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @ingroup cpl_image
   @brief  Split a complex image into its real and/or imaginary part(s)
   @param  im_real Pre-allocated image to hold the real part, or @c NULL
@@ -270,6 +278,7 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_fill_re_im)(cpl_image * im_real,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @ingroup cpl_image
   @brief  Fill an image with the real part of a complex image
   @param  im_real Pre-allocated image to hold the real part
@@ -313,6 +322,7 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_fill_re)(cpl_image * im_real,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @ingroup cpl_image
   @brief  Fill an image with the imaginary part of a complex image
   @param  im_imag Pre-allocated image to hold the imaginary part
@@ -356,6 +366,7 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_fill_im)(cpl_image * im_imag,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @ingroup cpl_image
   @brief  Split a complex image into its absolute and argument part(s)
   @param  im_abs Pre-allocated image to hold the absolute part, or @c NULL
@@ -422,6 +433,7 @@ cpl_error_code CPL_TYPE_ADD(cpl_image_fill_abs_arg)(cpl_image * im_abs,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @ingroup cpl_image
   @brief  Fill an image with the absolute part of a complex image
   @param  im_abs Pre-allocated image to hold the absolute part
@@ -465,6 +477,7 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_fill_abs)(cpl_image * im_abs,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @ingroup cpl_image
   @brief  Fill an image with the argument part of a complex image
   @param  im_arg Pre-allocated image to hold the argument part, or @c NULL
@@ -527,12 +540,13 @@ static cpl_error_code CPL_TYPE_ADD(cpl_image_conjugate)(cpl_image * self,
         (cpl_image_get_data)(self);
     const CPL_TYPE complex * pin = CPL_TYPE_ADD_COMPLEX_CONST
         (cpl_image_get_data)(other);
-    size_t i;
+    const size_t             nxy = (size_t)(self->nx * self->ny);
+    size_t                   i;
 
     cpl_ensure_code(pout != NULL, CPL_ERROR_INVALID_TYPE);
     cpl_ensure_code(pin  != NULL, CPL_ERROR_INVALID_TYPE);
 
-    for (i = 0; i < (size_t)(self->nx * self->ny); i++) {
+    for (i = 0; i < nxy; i++) {
         pout[i] = CPL_CONJ(pin[i]);
     }
 
