@@ -1,4 +1,4 @@
-/* $Id: cpl_image_io_impl.h,v 1.10 2012/03/13 09:31:58 llundin Exp $
+/* $Id: cpl_image_io_impl.h,v 1.11 2012/03/16 13:46:30 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/13 09:31:58 $
- * $Revision: 1.10 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/03/16 13:46:30 $
+ * $Revision: 1.11 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifndef CPL_IMAGE_IO_IMPL_H
@@ -43,6 +43,9 @@ CPL_BEGIN_DECLS
                             Function prototypes
  -----------------------------------------------------------------------------*/
 
+cpl_image * cpl_image_wrap_(cpl_size, cpl_size, cpl_type, void *)
+    CPL_ATTR_ALLOC;
+
 /* Special extractor functions for complex images */
 cpl_image * cpl_image_extract_real(const cpl_image *) CPL_ATTR_ALLOC;
 cpl_image * cpl_image_extract_imag(const cpl_image *) CPL_ATTR_ALLOC;
