@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic_body.h,v 1.35 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_imagelist_basic_body.h,v 1.39 2011/07/29 09:30:13 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -58,39 +58,39 @@
 
 #if CPL_OPERATION == CPL_IMLIST_BASIC_OPER
 
-    int i ;
+    cpl_size i;
 
     /* Check input image sets */
     cpl_ensure_code(in1, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(in2, CPL_ERROR_NULL_INPUT) ;
+    cpl_ensure_code(in2, CPL_ERROR_NULL_INPUT);
 
     /* Check image sets compatibility     */
     cpl_ensure_code( in1->ni == in2->ni, CPL_ERROR_ILLEGAL_INPUT);
 
     /* Loop on the planes and apply the operation */
-    for (i=0 ; i<in1->ni ; i++) {
+    for (i=0; i<in1->ni; i++) {
         const cpl_error_code error = 
             CPL_OPERATOR(in1->images[i], in2->images[i]);
         cpl_ensure_code(!error, error);
     }
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 
 #elif CPL_OPERATION == CPL_IMLIST_BASIC_IMAGE_LOCAL
 
-    int i ;
+    cpl_size i;
 
     /* Check input image sets */
     cpl_ensure_code(imlist, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(img,    CPL_ERROR_NULL_INPUT) ;
+    cpl_ensure_code(img,    CPL_ERROR_NULL_INPUT);
 
     /* Loop on the planes and apply the operation */
-    for (i=0 ; i<imlist->ni ; i++) {
+    for (i=0; i<imlist->ni; i++) {
         const cpl_error_code error = CPL_OPERATOR(imlist->images[i], img);
         cpl_ensure_code(!error, error);
     }
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 
 #elif CPL_OPERATION == CPL_IMLIST_BASIC_TIME_MEDIAN
 #ifndef L2_CACHE_BYTES
@@ -106,8 +106,8 @@
 #endif
     case CPL_TYPE_T:
     {
-        const int nz = imlist->ni;
-        CPL_TYPE ** pi = cpl_malloc(nz * sizeof(CPL_TYPE*));
+        const cpl_size nz = imlist->ni;
+        CPL_TYPE ** pi = cpl_malloc((size_t)nz * sizeof(CPL_TYPE*));
         CPL_TYPE *  po = CPL_IMAGE_GET_DATA(median);
         CPL_TYPE *  timeline = NULL;
 
@@ -115,21 +115,23 @@
             pi[k] = CPL_IMAGE_GET_DATA(imlist->images[k]);
 
 #if L2_CACHE_BYTES > 0
-        if (L2_CACHE_BYTES > 0 &&
-            sizeof(CPL_TYPE) * nz * (1 + nx * ny) > L2_CACHE_BYTES) {
+        if (L2_CACHE_BYTES > 0 && sizeof(CPL_TYPE) * (size_t)nz
+            * (size_t)(1 + nx * ny) > L2_CACHE_BYTES) {
             /* There is a cache available, and the entire imagelist does not
                fit inside it */
 
             /* Heuristics: When there are just a few planes, remapping does not
                pay off. It seems that break-even is at nz = nx / 15. */
-            const int max_aspect = 15;
+            const cpl_size max_aspect = 15;
 
             /* The largest number of pixels that can be handled by the cache */
-            const int mxl2 = (L2_CACHE_BYTES - sizeof(CPL_TYPE *) * nz)
-                / (sizeof(CPL_TYPE) * (1 + 2 * nz));
-            const int mxmax = mxl2 < nz * max_aspect ? mxl2 : nz * max_aspect;
-            int mx = nx;
-            int my = ny;
+            const cpl_size mxl2 = (L2_CACHE_BYTES
+                                   - (cpl_size)sizeof(CPL_TYPE *) * nz)
+                / ((cpl_size)sizeof(CPL_TYPE) * (1 + 2 * nz));
+            const cpl_size mxmax = mxl2 < nz * max_aspect
+                ? mxl2 : nz * max_aspect;
+            cpl_size mx = nx;
+            cpl_size my = ny;
 
 
             while ((mx & 1) == 0 && mx > mxmax) {
@@ -147,13 +149,14 @@
             }
 
 
-            if (sizeof(CPL_TYPE *) * nz + sizeof(CPL_TYPE) * mx * (1 + 2 * nz)
-              > L2_CACHE_BYTES/2) {
+            if (sizeof(CPL_TYPE *) * (size_t)nz + sizeof(CPL_TYPE)
+                * (size_t)mx * (size_t)(1 + 2 * nz) > L2_CACHE_BYTES/2) {
                 /* Heuristics: Too few planes may cause only a fraction of the
                    cache to be usable, in this case fall back on the simple
                    method. It seems that the break-even is about 50% cache
                    usage */
-                timeline = cpl_malloc(mx * nz * sizeof(CPL_TYPE));
+                timeline = cpl_malloc((size_t)mx * (size_t)nz
+                                      * sizeof(CPL_TYPE));
 
                 /* For each time line */
                 for (j=0; j < my; j++) {
@@ -172,7 +175,7 @@
 #endif
         if (timeline == NULL)  {
             /* Remap one pixel at a time */
-            timeline = cpl_malloc(nz * sizeof(CPL_TYPE));
+            timeline = cpl_malloc((size_t)nz * sizeof(CPL_TYPE));
             /* For each time line */
             for (i=0; i < nx * ny; i++) {
                 /* Get the pixels on the current time line  */
@@ -201,8 +204,8 @@
             = cpl_malloc((size_t)nx * (size_t)ny * sizeof(*pavg));
 
         /* A single timeline */
-        CPL_TYPE       *  ptime = cpl_malloc((size_t)ni * sizeof(*ptime));
-        int i, k;
+        CPL_TYPE * ptime = cpl_malloc((size_t)ni * sizeof(*ptime));
+        cpl_size   i, k;
 
 
         for (k = 0; k < ni; k++) {
@@ -225,7 +228,7 @@
             /* Compute the average */
             /* - using the recurrence relation from cpl_vector_get_mean() */
             for (k = 0; k < nuse; k++)
-                mean += ((double)ptime[nlow + k] - mean) / (k + 1);
+                mean += ((double)ptime[nlow + k] - mean) / (double)(k + 1);
 
             pavg[i] = (CPL_TYPE)mean;
         }
@@ -244,139 +247,159 @@
 
     case CPL_TYPE_T:
     {
-        CPL_TYPE       * pout_ima;
-        const CPL_TYPE * pcur_ima;
-        int              i, j, k; 
-     
-    cpl_vector      *   time_line ;
-    double          *   ptime_line ;
+        /* Need at least two values for the standard deviation */
+        const cpl_size notok = CX_MAX(1, (cpl_size)(keepfrac * (double)ni));
+        /* The pixel buffer of the output image */
+        CPL_TYPE * pclipped  = (CPL_TYPE*)cpl_malloc((size_t)nx * (size_t)ny *
+                                                     sizeof(*pclipped));
+        /* Temporary array for the non-clipped values */
+        double   * pvals = (double*)cpl_malloc((size_t)ni * sizeof(double));
+        cpl_size   i;
 
-        out_ima  = cpl_image_new(nx, ny, CPL_TYPE_T);
-        pout_ima = CPL_IMAGE_GET_DATA(out_ima);
-
-    /* Create the vector */
-    time_line  = cpl_vector_new(ni);
-    ptime_line = cpl_vector_get_data(time_line);
+        clipped  = cpl_image_wrap(nx, ny, CPL_TYPE_T, (void*)pclipped);
 
         /* Loop on the pixels */
-        for (j = 0; j < ny; j++) {
-            for (i = 0; i < nx; i++) {
-        double center, stdev ;
-        double low_thresh, high_thresh ;
-        int    nb_val = ni ;
-        int    old_nb_val = nb_val + 1; /* Only for the purpose of
-                           meeting the condition of
-                           the while the 1. time */
-
-        cpl_vector *  itimeline;
-        double     * pitimeline;
-        double mean = 0.0;
+        for (i = 0; i < nx * ny; i++) {
+            double   mean = 0.0;    /* Value to use when all pixels are bad */
+            cpl_size nok  = 0;      /* Number of values used in this timeline */
+            /* Need a 2nd counter, for convergence check and for contribution */
+            cpl_size prevok = ni + 1; /* No convergence before 1st iteration */
+            cpl_size k;
         
-                /* Fill the vector */
-                for (k = 0 ; k < ni ; k++) {
-                    cur_ima = cpl_imagelist_get_const(imlist, k);
-                    pcur_ima = CPL_IMAGE_GET_DATA_CONST(cur_ima);
-                    ptime_line[k] = (double)pcur_ima[i + j * nx];
+            /* Extract and count the non-bad values for this pixel */
+            for (k = 0; k < ni; k++) {
+                const cpl_image  * imgk = cpl_imagelist_get_const(self, k);
+                const cpl_mask   * bpm  = cpl_image_get_bpm_const(imgk);
+                const cpl_binary * bbpm = bpm ? cpl_mask_get_data_const(bpm)
+                    : NULL;
+
+                if (bbpm == NULL || !bbpm[i]) {
+                    const CPL_TYPE * pimg = CPL_IMAGE_GET_DATA_CONST(imgk);
+                    pvals[nok++] = (double)pimg[i];
+                }
+            }
+
+            /* Will not actually enter when there are too many bad pixels */
+            while (notok < nok && nok < prevok) {
+                /* cpl_tools_get_variancesum_double() is trusted to return
+                   a non-negative result even with rounding errors,
+                   so we assume it is safe to take the sqrt() below.
+                   Compute also the mean. Since it is computed prior
+                   to the clipping, it is the mean-value sought
+                   once the iteration has stopped. */
+                const double varsum = cpl_tools_get_variancesum_double
+                    (pvals, nok, &mean);
+
+                /* When computed, the median permutes the (non-clipped)
+                   values, but this has no effect on the result. */
+                const double center = mode == CPL_COLLAPSE_MEAN ||
+                    (mode == CPL_COLLAPSE_MEDIAN_MEAN && prevok < ni + 1)
+                    ? mean : cpl_tools_get_median_double(pvals, nok);
+
+                /* Compute the clipping thresholds */
+                const double stdev = sqrt(varsum / (double) (nok-1));
+                const double low_thresh  = center - kappalow  * stdev;
+                const double high_thresh = center + kappahigh * stdev;
+
+                prevok = nok;
+                /* Count the initial all-ok values */
+                for (nok = 0; nok < prevok; nok++) {
+                    /* Use exact same clipping criterion as next loop */
+                    if (!(low_thresh < pvals[nok] &&
+                          pvals[nok] < high_thresh)) {
+                        break;
+                    }
                 }
 
-        itimeline  = cpl_vector_duplicate(time_line);
-        pitimeline = cpl_vector_get_data(itimeline);
-
-        /* FIXME: Ensure that we always get inside this while loop */
-        while ((double)nb_val / (double)ni > keep &&
-               nb_val < old_nb_val) {
-            int * rejected = cpl_calloc(sizeof(int), nb_val);
-            int nrejected = 0;
-
-                    double median;
-
-            mean    = cpl_vector_get_mean(itimeline);
-            median  = cpl_vector_get_median(itimeline);
-
-            /* Get the mean and stdev */
-            center  = mode & CPL_COLLAPSE_MEAN ? mean : median;
-            stdev   = cpl_vector_get_stdev(itimeline);
-
-            low_thresh  = center - kappalow  * stdev;
-            high_thresh = center + kappahigh * stdev;
-
-            for (k = 0; k < nb_val; k++) {
-            if (pitimeline[k] > high_thresh && 
-                pitimeline[k] < low_thresh) {
-                rejected[k] = -1;
-                nrejected++;
-            } 
+                /* All values may have been checked above, but if not
+                   skip the one that cause the above loop to stop and
+                   check the rest and repack+count as necessary */
+                for (k = nok+1; k < prevok; k++) {
+                    if (low_thresh < pvals[k] &&
+                        pvals[k] < high_thresh) {
+                        pvals[nok++] = pvals[k];
+                    }
+                }
             }
 
-            k = 0;
-            while (k < nb_val) {
-            if (rejected[k] < 0)
-                pitimeline[k] = pitimeline[k + 1];
-            else
-                k++;
+            if (prevok == ni+1) {
+                /* The clipping loop was not entered at all */
+
+                prevok = nok; /* Needed for the contribution map */
+
+                /* The mean must be computed (without clipping) */
+                if (nok == 0) { /* All pixels are bad, so result pixel is bad */
+                    /* The call for the 1st bad pixel will create the bpm */
+                    cpl_mask   * bpm  = cpl_image_get_bpm(clipped);
+                    cpl_binary * bbpm = cpl_mask_get_data(bpm);
+
+                    bbpm[i] = CPL_BINARY_1;
+                } else {
+                    mean = cpl_tools_get_mean_double(pvals, nok);
+                }
             }
 
-            old_nb_val = nb_val;
-            nb_val -= nrejected;
-            cpl_vector_set_size(itimeline, nb_val);
+            /* Set the output image pixel value */
+            pclipped[i] = (CPL_TYPE)mean;
 
-            cpl_free(rejected);
+            if (pcontrib) {
+                /* The contributing number of values */
+                pcontrib[i] = (int)prevok;
+                if((cpl_size)pcontrib[i] != prevok) break;
+            }
+
+            cpl_tools_add_flops(6 + prevok);
 
-            if (nb_val <= 2) break;
         }
 
-                /* Set the output image pixel value */
-                pout_ima[i + j * nx] = (CPL_TYPE)mean;
+        cpl_free(pvals);
 
-        if (kept) 
-            pkept[i + j * nx] = old_nb_val;
-
-        cpl_vector_delete(itimeline);
-            }
+        if (i < nx * ny) {
+            cpl_image_delete(clipped);
+            clipped = NULL;
+            (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
         }
 
-    cpl_vector_delete(time_line);
-
-        break ;
+        break;
     }
 
 #elif CPL_OPERATION == CPL_IMLIST_BASIC_SWAP_AXIS 
 
     case CPL_TYPE_T:
     {
-        CPL_TYPE       * pcur_ima ;
-        const CPL_TYPE * pold_ima ;
+        CPL_TYPE       * pcur_ima;
+        const CPL_TYPE * pold_ima;
 
         /* SWAP X <-> Z */
         if (mode == CPL_SWAP_AXIS_XZ) {
-            for (i=0 ; i<nx ; i++) {
-                cur_ima=cpl_image_new(ni, ny, CPL_TYPE_T) ;
-                pcur_ima = CPL_IMAGE_GET_DATA(cur_ima) ;
-                for (j=0 ; j<ni ; j++) {
-                    old_ima = cpl_imagelist_get_const(ilist, j) ;
-                    pold_ima = CPL_IMAGE_GET_DATA_CONST(old_ima) ;
-                    for (k=0 ; k<ny ; k++) {
-                        pcur_ima[j+k*ni] = pold_ima[i+k*nx] ;
+            for (i=0; i<nx; i++) {
+                cur_ima=cpl_image_new(ni, ny, CPL_TYPE_T);
+                pcur_ima = CPL_IMAGE_GET_DATA(cur_ima);
+                for (j=0; j<ni; j++) {
+                    old_ima = cpl_imagelist_get_const(ilist, j);
+                    pold_ima = CPL_IMAGE_GET_DATA_CONST(old_ima);
+                    for (k=0; k<ny; k++) {
+                        pcur_ima[j+k*ni] = pold_ima[i+k*nx];
                     }
                 }
-                cpl_imagelist_set(swapped, cur_ima, i) ;
+                cpl_imagelist_set(swapped, cur_ima, i);
             }
         } else {
         /* SWAP Y <-> Z */
-            for (i=0 ; i<ny ; i++) {
-                cur_ima=cpl_image_new(nx, ni, CPL_TYPE_T) ;
-                pcur_ima = CPL_IMAGE_GET_DATA(cur_ima) ;
-                for (j=0 ; j<ni ; j++) {
-                    old_ima = cpl_imagelist_get_const(ilist, j) ;
-                    pold_ima = CPL_IMAGE_GET_DATA_CONST(old_ima) ;
-                    for (k=0 ; k<nx ; k++) {
-                        pcur_ima[k+j*nx] = pold_ima[k+i*nx] ;
+            for (i=0; i<ny; i++) {
+                cur_ima=cpl_image_new(nx, ni, CPL_TYPE_T);
+                pcur_ima = CPL_IMAGE_GET_DATA(cur_ima);
+                for (j=0; j<ni; j++) {
+                    old_ima = cpl_imagelist_get_const(ilist, j);
+                    pold_ima = CPL_IMAGE_GET_DATA_CONST(old_ima);
+                    for (k=0; k<nx; k++) {
+                        pcur_ima[k+j*nx] = pold_ima[k+i*nx];
                     }
                 }
-                cpl_imagelist_set(swapped, cur_ima, i) ;
+                cpl_imagelist_set(swapped, cur_ima, i);
             }
         }
-        break ;
+        break;
     }
 
 #endif
