@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_io.c,v 1.94 2011/12/16 15:39:57 llundin Exp $
+/* $Id: cpl_imagelist_io.c,v 1.97 2012/04/26 11:37:38 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/16 15:39:57 $
- * $Revision: 1.94 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/26 11:37:38 $
+ * $Revision: 1.97 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -375,7 +375,7 @@ cpl_image * cpl_imagelist_unset(cpl_imagelist * self,
   @brief    Empty an imagelist and deallocate all its images
   @param    self  The image list or NULL
   @return   Nothing
-  @see cpl_imagelist_delete()
+  @see  cpl_imagelist_empty(), cpl_imagelist_delete()
   @note If @em self is @c NULL nothing is done and no error is set.
 
   After the call the image list can be populated again. It must eventually
@@ -411,13 +411,34 @@ void cpl_imagelist_empty(cpl_imagelist * self)
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_imagelist
-  @brief    Free memory associated to a cpl_imagelist object.
+  @brief    Free memory used by a cpl_imagelist object, except the images
   @param    self    The image list or NULL
   @return   Nothing
-  @see      cpl_imagelist_empty()
-  @note If @em self is @c NULL nothing is done and no error is set.
+  @see cpl_imagelist_empty()
+  @note The caller must have pointers to all images in the list and is
+        reponsible for their deallocation. If @em self is @c NULL nothing is
+        done and no error is set.
+ */
+/*----------------------------------------------------------------------------*/
+void cpl_imagelist_unwrap(cpl_imagelist * self)
+{
 
-  The function deallocates the memory used by the imagelist @em self.
+    if (self != NULL) {
+
+        cpl_free(self->images);
+        cpl_free(self);
+    }
+
+    return;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @ingroup cpl_imagelist
+  @brief    Free all memory used by a cpl_imagelist object including the images
+  @param    self    The image list or NULL
+  @return   Nothing
+  @see      cpl_imagelist_empty(), cpl_imagelist_unwrap()
 
  */
 /*----------------------------------------------------------------------------*/
@@ -427,9 +448,7 @@ void cpl_imagelist_delete(cpl_imagelist * self)
     if (self != NULL) {
 
         cpl_imagelist_empty(self);
-
-        cpl_free(self->images);
-        cpl_free(self);
+        cpl_imagelist_unwrap(self);
     }
 
     return;
@@ -703,7 +722,7 @@ cpl_error_code cpl_imagelist_save(const cpl_imagelist    * self,
             return cpl_error_set_(CPL_ERROR_UNSPECIFIED);
         }
     } else {
-        /* Create the imagelist in the file */
+        /* Create the imagelist in a new HDU appended to the file */
         CPL_FITSIO_CREATE_IMG(fptr, ibpp, 3, (CPL_FITSIO_TYPE*)naxes, &error);
 
         if (mode & CPL_IO_CREATE)
