@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_io.h,v 1.28 2011/08/12 11:31:37 llundin Exp $
+/* $Id: cpl_imagelist_io.h,v 1.29 2012/04/04 12:26:54 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/08/12 11:31:37 $
- * $Revision: 1.28 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/04 12:26:54 $
+ * $Revision: 1.29 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifndef CPL_IMAGELIST_IO_H
@@ -59,7 +59,7 @@ cpl_image * cpl_imagelist_unset(cpl_imagelist *, cpl_size);
 
 /* Imagelist destructor */
 void cpl_imagelist_delete(cpl_imagelist *);
-
+void cpl_imagelist_unwrap(cpl_imagelist *);
 void cpl_imagelist_empty(cpl_imagelist *);
 
 /* Others */
