@@ -1,4 +1,4 @@
-/* $Id: cpl_init.c,v 1.44 2012/03/19 12:34:36 llundin Exp $
+/* $Id: cpl_init.c,v 1.47 2012/05/02 10:13:19 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/19 12:34:36 $
- * $Revision: 1.44 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/02 10:13:19 $
+ * $Revision: 1.47 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +35,7 @@
 #include "cpl_error_impl.h"
 #include "cpl_msg.h"
 #include "cpl_tools.h"
-#include "cpl_io_fits.h"
+#include "cpl_fits.h"
 
 #include <fitsio.h>
 
@@ -79,6 +79,7 @@
  * @param self  @em CPL_INIT_DEFAULT is the only supported value
  * @return Nothing.
  * @note The function must be called once before any other CPL function.
+ * @see cpl_fits_set_mode()
  *
  * This function sets up the library internal subsystems, which other 
  * CPL functions expect to be in a defined state. In particular, the CPL
@@ -100,22 +101,8 @@
  * unchanged.
  * 
  * This function also reads the environment variable @em CPL_IO_MODE.
- * If set to 1, CPL will use a new, somewhat experimental and optimized scheme
- * for FITS I/O. Normally, when a FITS file is processed with a CPL call, the
- * file is openened and closed during that call. However, repeated calls on
- * FITS data with many extensions causes the FITS headers to be parsed many
- * times, which can lead to a significant performance penalty. If instead
- * CPL_IO_MODE is set to 1, CPL will use internal storage to keep the FITS
- * files open between calls, and only close them when cpl_end() is called.
- * Also, certain FITS operations such as completely replacing the contents
- * of a FITS file that has already been read will cause the file to be closed
- * and opened. This optimized I/O mode cannot be used if the CPL application
- * accesses a FITS file without using CPL. En incomplete list of such access
- * is: calls to rename(), unlink(), fopen() and access with other software
- * started with for example system(). CPL_IO_MODE may be used in a multi-
- * threaded environment to the extent that the underlying FITS library
- * (CFITSIO) supports it.
- * 
+ * Iff set to 1, cpl_fits_set_mode() is called with CPL_FITS_START_CACHING.
+ *
  * Possible #_cpl_error_code_ set in this function:
  * - CPL_ERROR_INCOMPATIBLE_INPUT if there is an inconsistency between the run-
  *   time and compile-time versions of a library that CPL depends on internally,
@@ -137,7 +124,8 @@ cpl_init(unsigned self)
 #define CPL_CFITSIO_VERSION CFITSIO_VERSION
 #else
     /* This macro was introduced after version 2.510 */
-    /* Since CPL currently depends on version 2.510, it is
+    /* Since CPL was only verified to work with one version of CFITSIO that
+       did not define CFITSIO_VERSION (namely version 2.510), it is
        assumed at this point that the version actually is 2.510. */
 #define CPL_CFITSIO_VERSION 2.51
 #endif
@@ -260,7 +248,9 @@ cpl_init(unsigned self)
     cpl_error_init_locks();
 #endif
 
-    cpl_io_fits_init(use_io_fits);
+    if (use_io_fits) {
+        cpl_fits_set_mode(CPL_FITS_START_CACHING);
+    }
 
     return;
 }
@@ -353,7 +343,7 @@ void
 cpl_end(void)
 {
 
-    (void)cpl_io_fits_end();
+    (void)cpl_fits_set_mode(CPL_FITS_STOP_CACHING);
 
 #ifdef CPL_FFTWF_INSTALLED
     fftwf_cleanup();
