@@ -1,4 +1,4 @@
-/* $Id: cpl_plot.c,v 1.19 2011/07/21 08:35:14 llundin Exp $
+/* $Id: cpl_plot.c,v 1.22 2012/05/21 12:40:05 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/21 08:35:14 $
- * $Revision: 1.19 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/21 12:40:05 $
+ * $Revision: 1.22 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +55,7 @@
 
 /*----------------------------------------------------------------------------*/
 /**
- * @defgroup cpl_plot   Plotting functionalities
+ * @defgroup cpl_plot   Plotting of CPL objects
  *
  * This module provides functions to plot basic CPL objects
  * 
@@ -288,6 +288,8 @@ cpl_error_code cpl_plot_vectors(const char          *   pre,
                                 const cpl_vector    **  vectors,
                                 cpl_size                nvec)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
+
     const double    *   pvecx;
     const double    *   pvecy;
     cpl_size            vec_size = 0; /* Avoid uninit warning */
@@ -355,7 +357,7 @@ cpl_error_code cpl_plot_vectors(const char          *   pre,
         if (i==1) {
             /* Plot the first signal */
             sval = cpl_sprintf("plot '%s' %s;\n", names[i-1],
-                                                 options);
+                                                 options ? options : "");
         } else {
             sval = cpl_sprintf("replot '%s' t 'Vector %" CPL_SIZE_FORMAT
                                "' w lines;\n", names[i-1], i);
@@ -367,7 +369,9 @@ cpl_error_code cpl_plot_vectors(const char          *   pre,
     for (i=1; i<nvec; i++) remove(names[i-1]); 
     for (i=1; i<nvec; i++) cpl_free(names[i-1]); 
     cpl_free(names); 
-    return cpl_error_get_code();
+
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -404,6 +408,8 @@ cpl_error_code cpl_plot_bivectors(const char         *  pre,
                                   const cpl_bivector ** bivectors,
                                   cpl_size              nbvec)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
+
     const double * pvecx;
     const double * pvecy;
     char        ** names;
@@ -471,7 +477,9 @@ cpl_error_code cpl_plot_bivectors(const char         *  pre,
     for (i=0; i<nbvec; i++) remove(names[i]);
     for (i=0; i<nbvec; i++) cpl_free(names[i]);
     cpl_free(names);
-    return cpl_error_get_code();
+
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -776,6 +784,7 @@ cpl_error_code cpl_plot_image_row(const char      *   pre,
                                   cpl_size            lastrow,
                                   cpl_size            rowstep)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
     cpl_size nx, ny;
 
     cpl_ensure_code(image != NULL, CPL_ERROR_NULL_INPUT);
@@ -860,7 +869,8 @@ cpl_error_code cpl_plot_image_row(const char      *   pre,
 
     } while (0);
 
-    return cpl_error_get_code();
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -898,7 +908,9 @@ cpl_error_code cpl_plot_image_col(const char      * pre,
                                   cpl_size          lastcol,
                                   cpl_size          colstep)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
     cpl_size nx, ny;
+
     cpl_ensure_code(image != NULL, CPL_ERROR_NULL_INPUT);
 
     nx = cpl_image_get_size_x(image);
@@ -979,7 +991,8 @@ cpl_error_code cpl_plot_image_col(const char      * pre,
 
     } while (0);
 
-    return cpl_error_get_code();
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1015,6 +1028,7 @@ cpl_error_code cpl_plot_column(
         const char      *   xlab, 
         const char      *   ylab)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
     cpl_size            n;
     cpl_table       *   tempx;
     cpl_table       *   tempy;
@@ -1054,8 +1068,7 @@ cpl_error_code cpl_plot_column(
     cpl_ensure_code(type_y == CPL_TYPE_INT ||
             type_y == CPL_TYPE_FLOAT ||
             type_y == CPL_TYPE_DOUBLE, CPL_ERROR_INVALID_TYPE);
-    if (invalid_x == n || invalid_y == n) return cpl_error_get_code();
-
+    cpl_ensure_code(invalid_x < n && invalid_y < n, CPL_ERROR_DATA_NOT_FOUND);
   
     /* Cast columns to CPL_TYPE_DOUBLE and remove invalid entries */
     if (type_y != CPL_TYPE_DOUBLE || invalid_y > 0) {
@@ -1120,7 +1133,8 @@ cpl_error_code cpl_plot_column(
     if (tempx != NULL) cpl_table_delete(tempx);
     if (tempy != NULL) cpl_table_delete(tempy);
 
-    return cpl_error_get_code();
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 
 
@@ -1156,6 +1170,7 @@ cpl_error_code cpl_plot_columns(const char      *  pre,
                                 const char      ** labels, 
                                 cpl_size           nlabels)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
     cpl_size            n;
     cpl_table       *   tempx;
     cpl_table       *   tempy;
@@ -1265,7 +1280,7 @@ cpl_error_code cpl_plot_columns(const char      *  pre,
         if (i==1) {
             /* Plot the first signal */
             sval = cpl_sprintf("plot '%s' %s;\n", names[i-1],
-                                                 options);
+                                                 options ? options : "");
         } else {
             sval = cpl_sprintf("replot '%s' t 'Column %" CPL_SIZE_FORMAT
                                "' w lines;\n", names[i-1], i);
@@ -1280,7 +1295,9 @@ cpl_error_code cpl_plot_columns(const char      *  pre,
     cpl_free(names);
     cpl_table_delete(tempx);
     cpl_table_delete(tempy);
-    return cpl_error_get_code();
+
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 
 /**@}*/
