@@ -1,4 +1,4 @@
-/* $Id: cpl_stats.c,v 1.55 2012/02/15 16:20:35 llundin Exp $
+/* $Id: cpl_stats.c,v 1.56 2012/04/04 09:46:12 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/15 16:20:35 $
- * $Revision: 1.55 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/04 09:46:12 $
+ * $Revision: 1.56 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -487,6 +487,10 @@ cpl_stats * cpl_stats_new_from_image_window(
         cpl_size            urx,
         cpl_size            ury)
 {
+    const size_t llxsz = (size_t)llx;
+    const size_t llysz = (size_t)lly;
+    const size_t urxsz = (size_t)urx;
+    const size_t urysz = (size_t)ury;
     cpl_stats    * self;
     double         pix_sum = 0.0;
     double         sqr_sum = 0.0;
@@ -517,15 +521,18 @@ cpl_stats * cpl_stats_new_from_image_window(
     cpl_size            nbadpix;
     /* A map of the the bad pixels in the input */
     const cpl_binary  * badmap;
-    cpl_size            i, j;
+    size_t              i, j;
 
     /* Test inputs */
-    cpl_ensure(image != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(llx > 0 && llx <= image->nx && urx > 0 && urx <= image->nx,
-           CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
-    cpl_ensure(lly > 0 && lly <= image->ny && ury > 0 && ury <= image->ny,
-           CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
-    cpl_ensure(urx >= llx && ury >= lly, CPL_ERROR_ILLEGAL_INPUT, NULL);
+    cpl_ensure(image != NULL,    CPL_ERROR_NULL_INPUT,          NULL);
+
+    cpl_ensure(llx >  0,         CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+    cpl_ensure(lly >  0,         CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+    cpl_ensure(urx <= image->nx, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+    cpl_ensure(ury <= image->ny, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+    cpl_ensure(llx <= urx,       CPL_ERROR_ILLEGAL_INPUT,       NULL);
+    cpl_ensure(lly <= ury,       CPL_ERROR_ILLEGAL_INPUT,       NULL);
+
     cpl_ensure(mode != 1, CPL_ERROR_INVALID_TYPE, NULL);
     cpl_ensure(mode != 0, CPL_ERROR_UNSUPPORTED_MODE, NULL);
     cpl_ensure((mode & ~CPL_STATS_ALL) == 0, CPL_ERROR_UNSUPPORTED_MODE,
@@ -541,20 +548,14 @@ cpl_stats * cpl_stats_new_from_image_window(
     nbadpix = 0;
     if (badmap != NULL) {
         const cpl_binary * bpmj1 = badmap + (lly-1) * image->nx - 1;
-        size_t urxsz = (size_t)urx;
-        size_t urysz = (size_t)ury;
-        size_t isz, jsz;
-
-        cpl_ensure((cpl_size)urxsz == urx, CPL_ERROR_UNSUPPORTED_MODE, NULL);
-        cpl_ensure((cpl_size)urysz == ury, CPL_ERROR_UNSUPPORTED_MODE, NULL);
 
         firstgoodpos = -1;
-        for (jsz = (size_t)lly; jsz < 1 + urysz; jsz++, bpmj1 += image->nx) {
-            for (isz = (size_t)llx; isz < 1 + urxsz; isz++) {
-                if (bpmj1[isz] == CPL_BINARY_1) {
+        for (j = llysz; j < 1 + urysz; j++, bpmj1 += image->nx) {
+            for (i = llxsz; i < 1 + urxsz; i++) {
+                if (bpmj1[i] == CPL_BINARY_1) {
                     nbadpix++;
                 } else if (firstgoodpos < 0) {
-                    firstgoodpos = (isz - 1) + (jsz - 1) * image->nx;
+                    firstgoodpos = (i - 1) + (j - 1) * image->nx;
                 }
             }
         }
