@@ -1,4 +1,4 @@
-/* $Id: cpl_fits-test.c,v 1.19 2012/03/05 13:46:41 llundin Exp $
+/* $Id: cpl_fits-test.c,v 1.22 2012/05/11 13:21:03 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,23 +20,28 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/05 13:46:41 $
- * $Revision: 1.19 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/11 13:21:03 $
+ * $Revision: 1.22 $
+ * $Name: cpl-6_1_1 $
  */
 
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <math.h>
-#include <assert.h>
-
 #include "cpl_fits.h"
 #include "cpl_memory.h"
-#include "cpl_propertylist.h"
 #include "cpl_test.h"
-#include "cpl_propertylist.h"
+
+/* Needed for CPL_IO_FITS_MAX_OPEN */
+#include "cpl_io_fits.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+/*-----------------------------------------------------------------------------
+                                  Defines
+ -----------------------------------------------------------------------------*/
 
 #define CPL_FITS_NAME "cpl_fits-test.fits"
 
@@ -44,6 +49,12 @@
 #define CPL_FITS_NEXT 12
 #endif
 
+/*----------------------------------------------------------------------------
+                 Private function prototypes
+ ----------------------------------------------------------------------------*/
+
+static void cpl_fits_mode_test(void);
+
 /*-----------------------------------------------------------------------------
                                   Main
  -----------------------------------------------------------------------------*/
@@ -59,6 +70,8 @@ int main(void)
 
     /* Insert tests below */
 
+    cpl_fits_mode_test();
+
     ivalue = cpl_fits_count_extensions(NULL);
     cpl_test_error(CPL_ERROR_NULL_INPUT);
     cpl_test_eq(ivalue, -1);
@@ -144,3 +157,71 @@ int main(void)
 
     return cpl_test_end(0);
 }
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Test the CPL function(s)
+  @return   void
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_fits_mode_test(void)
+{
+    cpl_error_code code;
+    cpl_fits_mode mode, modec;
+
+#if CPL_IO_FITS_MAX_OPEN > 0
+    cpl_fits_mode modeo;
+
+    /* Copied from cpl_init() */
+    const char * io_fits_mode_string = getenv("CPL_IO_MODE");
+    const cpl_boolean use_io_fits = io_fits_mode_string != NULL &&
+        strcmp("1", io_fits_mode_string) == 0;
+#else
+    const cpl_boolean use_io_fits = CPL_FALSE;
+#endif
+
+    /* Get the mode */
+    mode = cpl_fits_get_mode();
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_eq(mode, use_io_fits ? CPL_FITS_START_CACHING
+                : CPL_FITS_STOP_CACHING);
+
+    /* Set the mode to what it already is */
+    code = cpl_fits_set_mode(mode);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    /* Verify that it did not change */
+    modec = cpl_fits_get_mode();
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_eq(mode, modec);
+
+#if CPL_IO_FITS_MAX_OPEN > 0
+    /* Change to the other mode */
+    modeo = use_io_fits ? CPL_FITS_STOP_CACHING : CPL_FITS_START_CACHING;
+    code = cpl_fits_set_mode(modeo);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    /* Verify that it did change */
+    modec = cpl_fits_get_mode();
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_eq(modeo, modec);
+#endif
+
+    /* Change back */
+    code = cpl_fits_set_mode(mode);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    /* Verify that it did change */
+    modec = cpl_fits_get_mode();
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_eq(mode, modec);
+
+    /* Test handling of incorrect mode combination */
+    code = cpl_fits_set_mode(CPL_FITS_STOP_CACHING & CPL_FITS_START_CACHING);
+    cpl_test_eq_error(code, CPL_ERROR_ILLEGAL_INPUT);
+
+    code = cpl_fits_set_mode(CPL_FITS_STOP_CACHING || CPL_FITS_START_CACHING);
+    cpl_test_eq_error(code, CPL_ERROR_INVALID_TYPE);
+
+}
