@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic-test.c,v 1.93 2012/02/15 15:56:49 llundin Exp $
+/* $Id: cpl_image_basic-test.c,v 1.94 2012/03/21 09:50:53 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/15 15:56:49 $
- * $Revision: 1.93 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/03/21 09:50:53 $
+ * $Revision: 1.94 $
+ * $Name: cpl-6_1_1 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -393,7 +393,7 @@ int main(void)
     cpl_msg_info("", "FFT(%g) round-off [DBL_EPSILON]: %g < %d", value,
                   err/DBL_EPSILON, IMAGESZ*IMAGESZ);
 
-    cpl_test_leq( fabs(err), IMAGESZ*IMAGESZ * DBL_EPSILON/1.5);
+    cpl_test_leq( fabs(err), IMAGESZ*IMAGESZ * DBL_EPSILON);
 
     cpl_image_delete(imd1);
 
