@@ -1,4 +1,4 @@
-/* $Id: cpl_propertylist-test.c,v 1.79 2011/04/18 11:31:00 llundin Exp $
+/* $Id: cpl_propertylist-test.c,v 1.80 2012/04/05 08:49:08 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/04/18 11:31:00 $
- * $Revision: 1.79 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/05 08:49:08 $
+ * $Revision: 1.80 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +41,7 @@
 #include "cpl_memory.h"
 
 #include "cpl_fits.h"
+#include "cpl_io_fits.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -1757,6 +1758,11 @@ int main(void)
     cpl_test_eq(cpl_propertylist_get_type(plist, "BADKEY"), CPL_TYPE_STRING);
     cpl_propertylist_delete(plist);
 #else
+
+    /* Need to close file due to non CPL access */
+    cpl_test_zero(cpl_io_fits_close(BASE "_26.fits", &status));
+    cpl_test_zero(status);
+
     cpl_test_eq(strlen(BADNUM), 3); /* For the below substitution to work */
     /* Remove quotes from value in FITS card */
     if (system("perl -pi -e 's/." BADNUM "     ./ " BADNUM "      /' "
@@ -1788,6 +1794,10 @@ int main(void)
 
     cpl_test_eq(cpl_propertylist_get_type(plist, "BADKEY"), CPL_TYPE_STRING);
 #else
+    /* Need to close file due to non CPL access */
+    cpl_test_zero(cpl_io_fits_close(BASE "_26.fits", &status));
+    cpl_test_zero(status);
+
     cpl_test_eq(strlen(COMPLEXVAL), 7); /* For the below substitution to work */
     /* Replace quotes with parenthesis to form a complex number */
     cpl_test_zero(system("perl -pi -e 's/\\047" COMPLEXVAL " \\047/("
