@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs.c,v 1.130 2012/02/14 14:38:17 llundin Exp $
+/* $Id: cpl_dfs.c,v 1.133 2012/05/11 13:27:12 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/14 14:38:17 $
- * $Revision: 1.130 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/11 13:27:12 $
+ * $Revision: 1.133 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1106,16 +1106,9 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_dfs_update_product_header(cpl_frameset * self)
 {
-
-    const cpl_error_code error = cpl_dfs_update_product_header_(self);
-    int status = 0;
-
-    if (cpl_io_fits_close_all(&status)) {
-        return cpl_error_set_fits(CPL_ERROR_FILE_IO, status, fits_close_file,
-                                  "Could not close all FITS files");
-    }
-
-    return error ? cpl_error_set_where_() : CPL_ERROR_NONE;
+    return cpl_dfs_update_product_header_(self)
+        | cpl_io_fits_close_tid(CPL_IO_FITS_ALL) /* FIXME: Close only writers */
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
 /**@}*/
@@ -1162,6 +1155,14 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                                               "filename='%s'", filename);
                 }
 
+                /* The previous call may be reusing file handle opened for
+                   previous I/O, so the file pointer needs to be moved */
+                if (fits_movabs_hdu(fproduct, 1, NULL, &error)) {
+                    code = cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
+                                              fits_movabs_hdu,
+                                              "filename='%s'", filename);
+                }
+
                 if (fits_read_card(fproduct, DATAMD5, card, &error)) {
                     code = cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
                                               fits_read_card, "filename='%s', "
@@ -1416,6 +1417,15 @@ static int cpl_is_fits(const char * self)
             (void)cpl_error_set_fits(CPL_ERROR_FILE_NOT_FOUND, error,
                                      fits_open_diskfile, "filename='%s'", self);
         error = 0;
+    } else if (fits_movabs_hdu(fptr, 1, NULL, &error)) {
+        /* The open call may be reusing file handle opened for
+           previous I/O, so the file pointer needs to be moved */
+        is_fits = error == FILE_NOT_OPENED ? -2 : 0;
+
+        if (is_fits < 0)
+            (void)cpl_error_set_fits(CPL_ERROR_FILE_NOT_FOUND, error,
+                                     fits_movabs_hdu, "filename='%s'", self);
+        error = 0;
     } else if (fits_read_imghdr(fptr, 0,  NULL, NULL, NULL, NULL, NULL, NULL,
                                 NULL, &error)) {
         is_fits = error == FILE_NOT_OPENED ? -2 : 0;
