@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs-test.c,v 1.60 2012/02/14 14:38:17 llundin Exp $
+/* $Id: cpl_dfs-test.c,v 1.62 2012/04/05 08:48:50 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/14 14:38:17 $
- * $Revision: 1.60 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/05 08:48:50 $
+ * $Revision: 1.62 $
+ * $Name: cpl-6_1_1 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -40,6 +40,8 @@
 #include "cpl_test.h"
 #include "cpl_image.h"
 
+#include "cpl_io_fits.h"
+
 /*-----------------------------------------------------------------------------
                                    Defines
  -----------------------------------------------------------------------------*/
@@ -103,6 +105,7 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     cpl_error_code      error;
     const int           next  = IMAGE_NEXT;
+    int                 fstatus = 0; /* CFITSIO error status */
     int                 i;
 
 
@@ -242,8 +245,8 @@ static void cpl_dfs_product_tests(const char * rawname)
     cpl_msg_debug("","Size of product header: %" CPL_SIZE_FORMAT,
                   cpl_propertylist_get_size(plist));
 
-    cpl_propertylist_append_char(plist, "QC MYCHAR", 42);
-    cpl_propertylist_set_comment(plist, "QC MYCHAR", "42");
+    cpl_propertylist_append_char(plist, "ESO QC MYCHAR", 42);
+    cpl_propertylist_set_comment(plist, "ESO QC MYCHAR", "42");
 
     error = cpl_dfs_save_paf("IIINSTRUME", "RRRECIPE", plist,
                              "cpl_dfs-test.paf");
@@ -281,6 +284,10 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     /* Test 6a: A failure on file format (missing file) */
 
+    /* Need to close file due to non CPL access */
+    cpl_test_zero(cpl_io_fits_close(CPL_DFS_FITSFILE, &fstatus));
+    cpl_test_zero(fstatus);
+
     /* Make sure file does not exist (from a previously failed test) */
     (void)remove(CPL_DFS_FITSFILE);
 
@@ -383,7 +390,6 @@ static void cpl_dfs_product_tests(const char * rawname)
     */
     cpl_test_eq_string(md5sum, "b4a11922757e107a2c10306867f52601");
 
-
     /* Test 7d: A successful call of cpl_dfs_update_product_header()
                 with a main HDU, one data-less extension and
                 one non-empty extension. */
