@@ -1,4 +1,4 @@
-/* $Id: cpl_apertures.c,v 1.30 2012/03/09 08:40:44 llundin Exp $
+/* $Id: cpl_apertures.c,v 1.33 2012/04/04 12:05:51 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/09 08:40:44 $
- * $Revision: 1.30 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/04 12:05:51 $
+ * $Revision: 1.33 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -233,7 +233,8 @@ void cpl_apertures_dump(const cpl_apertures * self,
 
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_TYPE_MISMATCH if lab is not of type CPL_TYPE_INT
+  - CPL_ERROR_TYPE_MISMATCH if lab is not of type CPL_TYPE_INT or
+                            if self is of a complex type 
   - CPL_ERROR_ILLEGAL_INPUT if lab has a negative value or zero maximum
   - CPL_ERROR_INCOMPATIBLE_INPUT if lab and self have different sizes.
   - CPL_ERROR_DATA_NOT_FOUND if one of the lab values is missing.
@@ -267,9 +268,9 @@ cpl_apertures * cpl_apertures_new_from_image(const cpl_image * self,
     cpl_ensure(cpl_image_get_type(lab) == CPL_TYPE_INT,
                CPL_ERROR_TYPE_MISMATCH, NULL);
     cpl_ensure(nx == cpl_image_get_size_x(lab),
-            CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
     cpl_ensure(ny == cpl_image_get_size_y(lab),
-            CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
 
     bpm = cpl_image_get_bpm_const(self);
     if (bpm != NULL && !cpl_mask_is_empty(bpm)) {
@@ -295,6 +296,13 @@ cpl_apertures * cpl_apertures_new_from_image(const cpl_image * self,
     /* Convert input image in a double image */
     dimage = cpl_image_get_type(self) == CPL_TYPE_DOUBLE ? self
         : (cimage = cpl_image_cast(self, CPL_TYPE_DOUBLE));
+
+    if (dimage == NULL) {
+        cpl_image_delete(labcopy);
+        (void)cpl_error_set_where_();
+        return NULL;
+    }
+
     pd = cpl_image_get_data_double_const(dimage);
 
     /* Create a cpl_apertures */
@@ -1389,16 +1397,65 @@ cpl_apertures * cpl_apertures_extract_window(const cpl_image  * self,
     return aperts;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Simple apertures creation from a user supplied selction mask
+  @param    self    The image to process
+  @param    selection  The mask of selected pixels
+  @return   The list of detected apertures or NULL if nothing detected or on
+            error.
+  @see cpl_image_labelise_mask_create(), cpl_apertures_new_from_image()
+
+  The values selected for inclusion in the apertures must have the non-zero
+  value in the selection mask, and must not be flagged as bad in the bad pixel
+  map of the image.
+
+  The input image type can be CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT or CPL_TYPE_INT.
+
+  Possible #_cpl_error_code_ set in this function:
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_INCOMPATIBLE_INPUT if self and selection have different sizes.
+  - CPL_ERROR_TYPE_MISMATCH if self is of a complex type 
+  - CPL_ERROR_DATA_NOT_FOUND if the selection mask is empty
+ */
+/*----------------------------------------------------------------------------*/
+cpl_apertures * cpl_apertures_extract_mask(const cpl_image * self,
+                                           const cpl_mask  * selection)
+{
+    cpl_errorstate  prestate = cpl_errorstate_get();
+    cpl_apertures * aperts = NULL;
+    cpl_size        nlabels;
+    /* Labelise the user provided selection */
+    cpl_image     * labels = cpl_image_labelise_mask_create(selection,
+                                                            &nlabels);
+
+    if (labels != NULL && nlabels > 0) {
+
+        /* Create the detected apertures list */
+        aperts = cpl_apertures_new_from_image(self, labels);
+
+    }
+
+    if (aperts == NULL) {
+        cpl_errorstate_is_equal(prestate)
+            ? (void)cpl_error_set_(CPL_ERROR_DATA_NOT_FOUND)
+            : (void)cpl_error_set_where_();
+    }
+
+    cpl_image_delete(labels);
+
+    return aperts;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Simple apertures detection in an image using a provided sigma
   @param    self    The image to process
   @param    sigma   Detection level
-  @return   The list of detected apertures or NULL if nothing detected or in
-            error case.
-  @note In order to avoid (potentially many) single-pixel detections the mask
-  of detected pixels is subjected to a 3x3 opening filter.
-  @see cpl_mask_filter()
+  @return   The list of detected apertures or NULL on error
+  @note In order to avoid (the potentially many) detections of small objects the
+  mask of detected pixels is subjected to a 3x3 morphological opening filter.
+  @see cpl_apertures_extract_mask(), cpl_mask_filter()
 
   The threshold used for the detection is the median plus the average distance
   to the median times sigma.
@@ -1407,8 +1464,9 @@ cpl_apertures * cpl_apertures_extract_window(const cpl_image  * self,
 
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_ILLEGAL_INPUT if sigma is negative or 0.0
-  - CPL_ERROR_ILLEGAL_OUTPUT if the image cannot be binarised or labelised
+  - CPL_ERROR_ILLEGAL_INPUT if sigma is non-positive
+  - CPL_ERROR_TYPE_MISMATCH if self is of a complex type
+  - CPL_ERROR_DATA_NOT_FOUND if the no apertures are found
  */
 /*----------------------------------------------------------------------------*/
 cpl_apertures * cpl_apertures_extract_sigma(const cpl_image * self,
@@ -1417,10 +1475,7 @@ cpl_apertures * cpl_apertures_extract_sigma(const cpl_image * self,
     double          median, med_dist;
     double          threshold;
     cpl_mask    *   selection;
-    cpl_mask    *   kernel;
-    cpl_image   *   labels;
-    cpl_apertures * aperts;
-    cpl_size        nlabels;
+    cpl_apertures * aperts = NULL;
 
     /* Check entries */
     cpl_ensure(self  != NULL, CPL_ERROR_NULL_INPUT,    NULL);
@@ -1431,46 +1486,33 @@ cpl_apertures * cpl_apertures_extract_sigma(const cpl_image * self,
     threshold = median + sigma * med_dist;
 
     /* Binarise the image */
-    selection=cpl_mask_threshold_image_create(self, threshold, DBL_MAX);
-    cpl_ensure(selection, CPL_ERROR_ILLEGAL_OUTPUT, NULL);
+    selection = cpl_mask_threshold_image_create(self, threshold, DBL_MAX);
+    if (selection != NULL) {
+        /* Use a morphological opening to remove the single pixel detections */
+        cpl_mask * kernel = cpl_mask_new(3, 3);
 
-    /* Apply a morphological opening to remove the single pixel detections */
-    kernel = cpl_mask_new(3, 3);
-    cpl_mask_not(kernel);
-    if (cpl_mask_filter(selection, selection, kernel, CPL_FILTER_OPENING,
-                        CPL_BORDER_ZERO) != CPL_ERROR_NONE) {
-        cpl_mask_delete(selection);
+        /* These two should not be able to fail */
+        const int  error  = cpl_mask_not(kernel) ||
+            cpl_mask_filter(selection, selection, kernel, CPL_FILTER_OPENING,
+                            CPL_BORDER_ZERO);
         cpl_mask_delete(kernel);
-        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
-        return NULL;
-    }
-    cpl_mask_delete(kernel);
 
-    /* Labelise the different detected apertures */
-    if ((labels = cpl_image_labelise_mask_create(selection, &nlabels))==NULL) {
+        if (!error) {
+            /* Create the detected apertures list */
+            aperts = cpl_apertures_extract_mask(self, selection);
+        }
+
         cpl_mask_delete(selection);
-        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
-        return NULL;
-    }
-    cpl_mask_delete(selection);
-
-    /* Nothing detected */
-    if (nlabels == 0) {
-        cpl_image_delete(labels);
-        return NULL;
     }
 
-    /* Create the detected apertures list */
-    if ((aperts = cpl_apertures_new_from_image(self, labels)) == NULL) {
-        cpl_image_delete(labels);
-        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
-        return NULL;
+    if (aperts == NULL) {
+        (void)cpl_error_set_where_();
     }
 
-    cpl_image_delete(labels);
     return aperts;
 }
 
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Compute FWHM values in x and y for a list of apertures
