@@ -1,4 +1,4 @@
-/* $Id: cpl_apertures.h,v 1.10 2011/12/20 09:11:42 llundin Exp $
+/* $Id: cpl_apertures.h,v 1.11 2012/04/04 12:05:51 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/20 09:11:42 $
- * $Revision: 1.10 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/04 12:05:51 $
+ * $Revision: 1.11 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifndef CPL_APERTURES_H
@@ -100,6 +100,8 @@ cpl_apertures * cpl_apertures_extract_window(const cpl_image *,
                                              cpl_size, cpl_size *)
     CPL_ATTR_ALLOC;
 
+cpl_apertures * cpl_apertures_extract_mask(const cpl_image *,
+                                           const cpl_mask *) CPL_ATTR_ALLOC;
 cpl_apertures * cpl_apertures_extract_sigma(const cpl_image *,
                                             double) CPL_ATTR_ALLOC;
 
