@@ -1,4 +1,4 @@
-/* $Id: cpl_detector.c,v 1.30 2011/07/29 12:03:46 llundin Exp $
+/* $Id: cpl_detector.c,v 1.32 2012/05/21 12:43:35 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,35 +20,21 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/29 12:03:46 $
- * $Revision: 1.30 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/21 12:43:35 $
+ * $Revision: 1.32 $
+ * $Name: cpl-6_1_1 $
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-/*-----------------------------------------------------------------------------
-                                   Define
- -----------------------------------------------------------------------------*/
-
-#define CPL_CLASS_NONE      0
-#define CPL_CLASS_DOUBLE    1
-#define CPL_CLASS_FLOAT     2
-#define CPL_CLASS_INT       3
-
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "cpl_detector.h"
 
-#include <cpl_msg.h>
-#include <cpl_stats.h>
-#include <cpl_mask.h>
-#include <cpl_matrix.h>
-#include <cpl_tools.h>
 #include <cpl_memory.h>
 #include <cpl_math_const.h>
 #include <cpl_error_impl.h>
@@ -70,8 +56,25 @@
 #define qdist(r1,t1,r2,t2) \
   ((r1*r1)+(r2*r2)-2*r1*r2*cos((t1-t2)))
 
+#define CPL_CLASS_NONE      0
+#define CPL_CLASS_DOUBLE    1
+#define CPL_CLASS_FLOAT     2
+#define CPL_CLASS_INT       3
+
+/*----------------------------------------------------------------------------*/
+/**
+ * @defgroup cpl_detector   High-level functions to compute detector features
+ *
+ * @par Synopsis:
+ * @code
+ *   #include "cpl_detector.h"
+ * @endcode
+ */
+/*----------------------------------------------------------------------------*/
+/**@{*/
+
 /*-----------------------------------------------------------------------------
-                            Function codes
+                   Private Function declarations and definitions
  -----------------------------------------------------------------------------*/
 
 /* Define the C-type dependent functions */
@@ -130,18 +133,6 @@ static cpl_bivector * cpl_bivector_gen_ring_poisson(const double *,
                                                     const cpl_size)
     CPL_ATTR_NONNULL CPL_ATTR_ALLOC;
 
-/*----------------------------------------------------------------------------*/
-/**
- * @defgroup cpl_detector   High-level functions to compute detector features
- *
- * @par Synopsis:
- * @code
- *   #include "cpl_detector.h"
- * @endcode
- */
-/*----------------------------------------------------------------------------*/
-/**@{*/
-
 /*-----------------------------------------------------------------------------
                             Function codes
  -----------------------------------------------------------------------------*/
@@ -239,6 +230,7 @@ static cpl_error_code cpl_flux_get_window(
         double          *   bias,
         double          *   bias_error)
 {
+    cpl_errorstate prestate = cpl_errorstate_get();
     const cpl_size hsize = ron_hsize < 0 ? RECT_RON_HS      : ron_hsize;
     const cpl_size nsamples = ron_nsamp < 0 ? RECT_RON_SAMPLES : ron_nsamp;
     cpl_bivector * sample_reg;
@@ -314,7 +306,8 @@ static cpl_error_code cpl_flux_get_window(
     cpl_vector_delete(rms_list);
     cpl_vector_delete(bias_list);
 
-    return cpl_error_get_code();
+    return cpl_errorstate_is_equal(prestate) ? CPL_ERROR_NONE
+        : cpl_error_set_where_();
 }
 #undef RECT_RON_HS
 #undef RECT_RON_SAMPLES
