@@ -1,4 +1,4 @@
-/* $Id: cpl_geom_img.c,v 1.49 2011/12/20 09:11:42 llundin Exp $
+/* $Id: cpl_geom_img.c,v 1.50 2012/04/04 12:26:54 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/20 09:11:42 $
- * $Revision: 1.49 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/04 12:26:54 $
+ * $Revision: 1.50 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -153,7 +153,6 @@ static void cpl_geom_img_offset_saa_all_float(cpl_image *,
 
 static cpl_imagelist * cpl_imagelist_wrap_all_but_first(cpl_imagelist *)
     CPL_ATTR_ALLOC;
-static void cpl_imagelist_unwrap(cpl_imagelist *);
 
 static cpl_bivector * cpl_bivector_wrap_all_but_first(cpl_bivector *)
     CPL_ATTR_ALLOC;
@@ -1142,28 +1141,6 @@ static cpl_imagelist * cpl_imagelist_wrap_all_but_first(cpl_imagelist * other)
     return self;
 }
 
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief    Shallow delete of all but the first image in a list
-  @param    self   The list to unwrap
-  @return void
-
- */
-/*----------------------------------------------------------------------------*/
-static void cpl_imagelist_unwrap(cpl_imagelist * self)
-{
-    if (self != NULL) {
-        cpl_size n = cpl_imagelist_get_size(self);
-
-        for (; n > 0; n--) {
-            (void)cpl_imagelist_unset(self, n-1);
-        }
-        cpl_imagelist_delete(self);
-    }
-}
-
-
 /*----------------------------------------------------------------------------*/
 /**
   @internal
