@@ -1,4 +1,4 @@
-/* $Id: cpl_geom_img_body.h,v 1.36 2010/11/26 15:10:00 llundin Exp $
+/* $Id: cpl_geom_img_body.h,v 1.39 2011/08/08 13:03:46 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -48,32 +48,32 @@ static
 void CPL_TYPE_ADD(cpl_geom_ima_offset_xcorr_subw)(double          * correl,
                                                   const cpl_image * im1,
                                                   const cpl_image * im2,
-                                                  int               x_1,
-                                                  int               y_1,
-                                                  int               x_2,
-                                                  int               y_2,
-                                                  int               s_hx,
-                                                  int               s_hy,
-                                                  int               m_hx,
-                                                  int               m_hy,
-                                                  int *             pk_min,
-                                                  int *             pl_min)
+                                                  cpl_size          x_1,
+                                                  cpl_size          y_1,
+                                                  cpl_size          x_2,
+                                                  cpl_size          y_2,
+                                                  cpl_size          s_hx,
+                                                  cpl_size          s_hy,
+                                                  cpl_size          m_hx,
+                                                  cpl_size          m_hy,
+                                                  cpl_size        * pk_min,
+                                                  cpl_size        * pl_min)
 
 {
     double           sqsum_min = DBL_MAX;
-    const int        nx = cpl_image_get_size_x(im1);
-    const int        ny = cpl_image_get_size_y(im1);
+    const cpl_size   nx = cpl_image_get_size_x(im1);
+    const cpl_size   ny = cpl_image_get_size_y(im1);
     /* Point to the center corellation position */
     const CPL_TYPE * pi1 = CPL_TYPE_ADD_CONST(cpl_image_get_data)(im1)
         + x_1 + y_1 * nx;
     const CPL_TYPE * pi2 = CPL_TYPE_ADD_CONST(cpl_image_get_data)(im2)
         + x_2 + y_2 * nx;
     double         * pxc = correl + s_hx; /* Offset for indexing with k */
-    const int        s_hx_min = -CX_MIN(s_hx, CX_MIN(x_1, x_2));
-    const int        s_hy_min = -CX_MIN(s_hy, CX_MIN(y_1, y_2));
-    const int        s_hx_max =  CX_MIN(s_hx, nx-1-CX_MAX(x_1, x_2));
-    const int        s_hy_max =  CX_MIN(s_hy, ny-1-CX_MAX(y_1, y_2));
-    int              k, l;
+    const cpl_size   s_hx_min = -CX_MIN(s_hx, CX_MIN(x_1, x_2));
+    const cpl_size   s_hy_min = -CX_MIN(s_hy, CX_MIN(y_1, y_2));
+    const cpl_size   s_hx_max =  CX_MIN(s_hx, nx-1-CX_MAX(x_1, x_2));
+    const cpl_size   s_hy_max =  CX_MIN(s_hy, ny-1-CX_MAX(y_1, y_2));
+    cpl_size         k, l;
 
 
     /* Outside the image(s) */
@@ -90,11 +90,11 @@ void CPL_TYPE_ADD(cpl_geom_ima_offset_xcorr_subw)(double          * correl,
             pxc[k] = -1.0;
         }
         for (; k <= s_hx_max; k++) {
-            const int m_hx_min = -CX_MIN(m_hx, CX_MIN(x_1 + k, x_2));
-            const int m_hy_min = -CX_MIN(m_hy, CX_MIN(y_1 + l, y_2));
-            const int m_hx_max =  CX_MIN(m_hx, nx-1-CX_MAX(x_1 + k, x_2));
-            const int m_hy_max =  CX_MIN(m_hy, ny-1-CX_MAX(y_1 + l, y_2));
-            const int npix = (1+m_hx_max-m_hx_min) * (1+m_hy_max-m_hy_min);
+            const cpl_size m_hx_min = -CX_MIN(m_hx, CX_MIN(x_1 + k, x_2));
+            const cpl_size m_hy_min = -CX_MIN(m_hy, CX_MIN(y_1 + l, y_2));
+            const cpl_size m_hx_max =  CX_MIN(m_hx, nx-1-CX_MAX(x_1 + k, x_2));
+            const cpl_size m_hy_max =  CX_MIN(m_hy, ny-1-CX_MAX(y_1 + l, y_2));
+            const cpl_size npix = (1+m_hx_max-m_hx_min) * (1+m_hy_max-m_hy_min);
 
             if (npix < m_hx + m_hy) {
                 /* FIXME: Arbitrary lower limit on sampled pixels */
@@ -104,7 +104,7 @@ void CPL_TYPE_ADD(cpl_geom_ima_offset_xcorr_subw)(double          * correl,
                 const double     rnpix = 1.0 / (double)npix;
                 const CPL_TYPE * pos1 = pi1 + k + (l + m_hy_min) * nx;
                 const CPL_TYPE * pos2 = pi2     + (    m_hy_min) * nx;
-                int              i, j;
+                cpl_size         i, j;
 
                 /* One sum of squared differences */
                 for (j = m_hy_min; j <= m_hy_max; j++, pos1 += nx, pos2 += nx) {
@@ -165,36 +165,40 @@ void CPL_TYPE_ADD(cpl_geom_ima_offset_xcorr_subw)(double          * correl,
 /*----------------------------------------------------------------------------*/
 static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
                                                   cpl_image * contrib,
-                                                  int rmin, int rmax,
+                                                  cpl_size rmin, cpl_size rmax,
                                                   double start_x,
                                                   double start_y,
                                                   const cpl_imagelist * ilist,
-                                                  int firstbpm,
+                                                  cpl_size firstbpm,
                                                   const cpl_bivector * offs,
                                                   const cpl_vector * xyprofile,
-                                                  int tabsperpix)
+                                                  cpl_size tabsperpix)
 
 {
 
-    const int           nima  = cpl_imagelist_get_size(ilist);
+    const cpl_size      nima  = cpl_imagelist_get_size(ilist);
     const cpl_image *   img1  = cpl_imagelist_get_const(ilist, 0);
-    const int           sizex = cpl_image_get_size_x(img1);
-    const int           sizey = cpl_image_get_size_y(img1);
-    const int           nx    = cpl_image_get_size_x(final);
-    const int           ny    = cpl_image_get_size_y(final);
-    const int           rtot = rmin + rmax;
+    const cpl_size      sizex = cpl_image_get_size_x(img1);
+    const cpl_size      sizey = cpl_image_get_size_y(img1);
+    const cpl_size      nx    = cpl_image_get_size_x(final);
+    const cpl_size      ny    = cpl_image_get_size_y(final);
+    const cpl_size      rtot = rmin + rmax;
     CPL_TYPE          * po = CPL_TYPE_ADD(cpl_image_get_data)(final);
     const double      * offs_x = cpl_bivector_get_x_data_const(offs);
     const double      * offs_y = cpl_bivector_get_y_data_const(offs);
-    const double      * interp_kernel = cpl_vector_get_data_const(xyprofile);
-    double            * acc = (double*)cpl_malloc(nx * nima * sizeof(double));
+    double            * acc = (double*)cpl_malloc((size_t)nx * (size_t)nima
+                                                  * sizeof(*acc));
     int               * pcontrib = cpl_image_get_data_int(contrib);
-    const CPL_TYPE   ** ppi = cpl_malloc(nima * sizeof(CPL_TYPE*));
+    const CPL_TYPE   ** ppi = cpl_malloc((size_t)nima * sizeof(CPL_TYPE*));
     const cpl_binary ** ppibpm = NULL;
-    int               * ncontrib = (int*)cpl_calloc(sizeof(int), nx);
-    int               * offset_ij = (int*)cpl_malloc(2 * nima * sizeof(int));
-    double            * rsc = (double*)cpl_malloc(nima * 8 * sizeof(double));
-    int                 j, p;
+    int               * ncontrib = (int*)cpl_calloc(sizeof(int), (size_t)nx);
+    cpl_size          * offset_ij = cpl_malloc(2 * (size_t)nima
+                                               * sizeof(*offset_ij));
+    double            * rsc = (double*)cpl_malloc((size_t)nima * 8
+                                                  * sizeof(*rsc));
+    cpl_boolean       * isint = (cpl_boolean*)cpl_calloc((size_t)nima,
+                                                         sizeof(*isint));
+    cpl_size            j, p;
     cpl_flops           myflop_count = 0;
 
     for (p=0; p < nima; p++) {
@@ -205,45 +209,55 @@ static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
         const double      suboff_y = offset_y - floor(offset_y);
 
         /* Which tabulated value index shall be used? */
-        const int         tabx = (int)(0.5 + suboff_x * (double)(tabsperpix));
-        const int         taby = (int)(0.5 + suboff_y * (double)(tabsperpix));
-
-        const double      sumrs
-            = (  interp_kernel[    tabsperpix + tabx]
-               + interp_kernel[                 tabx]
-               + interp_kernel[    tabsperpix - tabx]
-               + interp_kernel[2 * tabsperpix - tabx])
-            * (  interp_kernel[    tabsperpix + taby]
-               + interp_kernel[                 taby]
-               + interp_kernel[    tabsperpix - taby]
-               + interp_kernel[2 * tabsperpix - taby]);
+        const cpl_size    tabx = (int)(0.5 + suboff_x * (double)(tabsperpix));
+        const cpl_size    taby = (int)(0.5 + suboff_y * (double)(tabsperpix));
 
 
-        offset_ij[0 + p * 2] = (int)floor(offset_x);
-        offset_ij[1 + p * 2] = (int)floor(offset_y);
+        offset_ij[0 + p * 2] = (cpl_size)floor(offset_x);
+        offset_ij[1 + p * 2] = (cpl_size)floor(offset_y);
 
         ppi[p] = CPL_TYPE_ADD_CONST(cpl_image_get_data)
             (cpl_imagelist_get_const(ilist, p));
         ppi[p] += offset_ij[0 + p * 2];
 
-        /* Compute resampling coefficients */
-        /* rsc[0..3] in x, rsc[4..7] in y  */
-        /* Also divide the y-rsc with the sum of the rsc */
+        if (tabx == 0 && taby == 0) {
+            isint[p] = CPL_TRUE;
+        } else {
 
-        rsc[0 + p * 8] = interp_kernel[    tabsperpix + tabx];
-        rsc[1 + p * 8] = interp_kernel[                 tabx];
-        rsc[2 + p * 8] = interp_kernel[    tabsperpix - tabx];
-        rsc[3 + p * 8] = interp_kernel[2 * tabsperpix - tabx];
-        rsc[4 + p * 8] = interp_kernel[    tabsperpix + taby] / sumrs;
-        rsc[5 + p * 8] = interp_kernel[                 taby] / sumrs;
-        rsc[6 + p * 8] = interp_kernel[    tabsperpix - taby] / sumrs;
-        rsc[7 + p * 8] = interp_kernel[2 * tabsperpix - taby] / sumrs;
+            const double      * interp_kernel
+                = cpl_vector_get_data_const(xyprofile);
 
+            const double      sumrs
+                = (  interp_kernel[    tabsperpix + tabx]
+                     + interp_kernel[                 tabx]
+                     + interp_kernel[    tabsperpix - tabx]
+                     + interp_kernel[2 * tabsperpix - tabx])
+                * (  interp_kernel[    tabsperpix + taby]
+                     + interp_kernel[                 taby]
+                     + interp_kernel[    tabsperpix - taby]
+                     + interp_kernel[2 * tabsperpix - taby]);
+
+
+            /* Compute resampling coefficients */
+            /* rsc[0..3] in x, rsc[4..7] in y  */
+            /* Also divide the y-rsc with the sum of the rsc */
+
+            rsc[0 + p * 8] = interp_kernel[    tabsperpix + tabx];
+            rsc[1 + p * 8] = interp_kernel[                 tabx];
+            rsc[2 + p * 8] = interp_kernel[    tabsperpix - tabx];
+            rsc[3 + p * 8] = interp_kernel[2 * tabsperpix - tabx];
+            rsc[4 + p * 8] = interp_kernel[    tabsperpix + taby] / sumrs;
+            rsc[5 + p * 8] = interp_kernel[                 taby] / sumrs;
+            rsc[6 + p * 8] = interp_kernel[    tabsperpix - taby] / sumrs;
+            rsc[7 + p * 8] = interp_kernel[2 * tabsperpix - taby] / sumrs;
+
+        }
     }
 
     /* Get the masks if there are bad pixels  */
     if (firstbpm < nima) {
-        ppibpm = (const cpl_binary **)cpl_malloc(nima * sizeof(cpl_binary*));
+        ppibpm = (const cpl_binary **)cpl_malloc((size_t)nima
+                                                 * sizeof(cpl_binary*));
 
         for (p = 0; p < firstbpm; p++) {
             ppibpm[p] = NULL;
@@ -264,11 +278,12 @@ static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
 
     /* Triple loop */
     for (j=0; j < ny; j++) {
-        int i;
+        cpl_flops myintp_count = 0;
+        cpl_size  i;
 
         for (p=0; p < nima; p++) {
 
-            const int py = j + offset_ij[1 + p * 2];
+            const cpl_size py = j + offset_ij[1 + p * 2];
 
             /* If original pixel is present in the current plane */
             if (py > 1 && py < (sizey-2)) {
@@ -277,80 +292,86 @@ static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
                 const cpl_binary * pibpm = NULL;
                 
                 /* First offset source location used */
-                const int i0 = CX_MAX(0, 2 - offset_ij[0 + p * 2]);
+                const cpl_size i0 = CX_MAX(0, 2 - offset_ij[0 + p * 2]);
                 /* First offset source location not used */
-                const int i1 = CX_MIN(sizex - 2 - offset_ij[0 + p * 2], nx);
+                const cpl_size i1 = CX_MIN(sizex - 2 - offset_ij[0 + p * 2],
+                                           nx);
+
 
                 if(ppibpm != NULL && ppibpm[p] != NULL)
                     pibpm = ppibpm[p] + py * sizex;
+
+                if (isint[p]) {
+                    /* The shift is purely integer, so no interpolation done */
+                    /* When the whole list has integer shifts, the speed-up is
+                       a factor between 3 and 4 on the unit tests */
                 
-                for (i = i0; i < i1; i++) {
+                    for (i = i0; i < i1; i++) {
         
-                    /* assert( px == i + offset_ij[0 + p * 2] ); */
+                        /* assert( px == i + offset_ij[0 + p * 2] ); */
 
-                    /* Check if any of the pixels used for the interpolation
-                     * is a bad pixel. If true, this image is not used 
-                     * for this pixel */
-                    if(pibpm != NULL &&
-                       ((pibpm[i - 1 - sizex]     &&
-                         rsc[4 + 8*p] * rsc[0 + 8*p] != 0.0) ||
-                        (pibpm[i     - sizex]     &&
-                         rsc[4 + 8*p] * rsc[1 + 8*p] != 0.0) ||
-                        (pibpm[i + 1 - sizex]     &&
-                         rsc[4 + 8*p] * rsc[2 + 8*p] != 0.0) ||
-                        (pibpm[i + 2 - sizex]     &&
-                         rsc[4 + 8*p] * rsc[3 + 8*p] != 0.0) ||
-                        (pibpm[i - 1]             &&
-                         rsc[5 + 8*p] * rsc[0 + 8*p] != 0.0) ||
-                        (pibpm[i    ]             &&
-                         rsc[5 + 8*p] * rsc[1 + 8*p] != 0.0) ||
-                        (pibpm[i + 1]             &&
-                         rsc[5 + 8*p] * rsc[2 + 8*p] != 0.0) ||
-                        (pibpm[i + 2]             &&
-                         rsc[5 + 8*p] * rsc[3 + 8*p] != 0.0) ||
-                        (pibpm[i - 1 + sizex]     &&
-                         rsc[6 + 8*p] * rsc[0 + 8*p] != 0.0) ||
-                        (pibpm[i     + sizex]     &&
-                         rsc[6 + 8*p] * rsc[1 + 8*p] != 0.0) ||
-                        (pibpm[i + 1 + sizex]     &&
-                         rsc[6 + 8*p] * rsc[2 + 8*p] != 0.0) ||
-                        (pibpm[i + 2 + sizex]     &&
-                         rsc[6 + 8*p] * rsc[3 + 8*p] != 0.0) ||
-                        (pibpm[i - 1 + sizex * 2] &&
-                         rsc[7 + 8*p] * rsc[0 + 8*p] != 0.0) ||
-                        (pibpm[i     + sizex * 2] &&
-                         rsc[7 + 8*p] * rsc[1 + 8*p] != 0.0) ||
-                        (pibpm[i + 1 + sizex * 2] &&
-                         rsc[7 + 8*p] * rsc[2 + 8*p] != 0.0) ||
-                        (pibpm[i + 2 + sizex * 2] &&
-                         rsc[7 + 8*p] * rsc[3 + 8*p] != 0.0)))
-                        {
-                            continue;
+                        /* Check if the pixel used
+                         * is a bad pixel. If true, this image is not used 
+                         * for this pixel */
+                        if (pibpm == NULL || !pibpm[i]) {
+
+                            /* Collect pixel and update number of
+                               contributing pixels */
+                            /* acc[] is accessed with stride nima here, in order
+                               to achieve stride one access below */
+                            acc[ncontrib[i] + i * nima] = pi[i];
+                            ncontrib[i]++;
                         }
+                    }
+                } else {
+                
+                    for (i = i0; i < i1; i++) {
+        
+                        /* assert( px == i + offset_ij[0 + p * 2] ); */
 
-                    /* Compute interpolated pixel and update number of
-                       contributing pixels */
-                    /* acc[] is accessed with stride nima here, in order
-                       to achieve stride one access below */
-                    acc[ncontrib[i] + i * nima] =
-                        rsc[4 + 8*p] * (  rsc[0 + 8*p] * pi[i - 1 - sizex] +
-                                          rsc[1 + 8*p] * pi[i     - sizex] +
-                                          rsc[2 + 8*p] * pi[i + 1 - sizex] +
-                                          rsc[3 + 8*p] * pi[i + 2 - sizex] ) +
-                        rsc[5 + 8*p] * (  rsc[0 + 8*p] * pi[i - 1] +
-                                          rsc[1 + 8*p] * pi[i    ] +
-                                          rsc[2 + 8*p] * pi[i + 1] +
-                                          rsc[3 + 8*p] * pi[i + 2] ) +
-                        rsc[6 + 8*p] * (  rsc[0 + 8*p] * pi[i - 1 + sizex] +
-                                          rsc[1 + 8*p] * pi[i     + sizex] +
-                                          rsc[2 + 8*p] * pi[i + 1 + sizex] +
-                                          rsc[3 + 8*p] * pi[i + 2 + sizex] ) +
-                        rsc[7 + 8*p] * (  rsc[0 + 8*p] * pi[i - 1 + sizex * 2] +
-                                          rsc[1 + 8*p] * pi[i     + sizex * 2] +
-                                          rsc[2 + 8*p] * pi[i + 1 + sizex * 2] +
-                                          rsc[3 + 8*p] * pi[i + 2 + sizex * 2] );
+                        /* Check if any of the pixels used for the interpolation
+                         * is a bad pixel. If true, this image is not used 
+                         * for this pixel */
+                        /* An earlier check required the pixel to be
+                           bad and the resampling coefficient to be non-zero.
+                           This was dropped to support bad pixels w. NaNs */ 
+                        if(pibpm == NULL ||
+                           /* One read for all 4 pixels in the bpm row */
+                           (!*((const uint32_t*)(pibpm + i - 1 - sizex    )) &&
+                            !*((const uint32_t*)(pibpm + i - 1            )) &&
+                            !*((const uint32_t*)(pibpm + i - 1 + sizex    )) &&
+                            !*((const uint32_t*)(pibpm + i - 1 + sizex * 2)))) {
 
-                    ncontrib[i]++;
+                            /* Compute interpolated pixel and update number of
+                               contributing pixels */
+                            /* acc[] is accessed with stride nima here, in order
+                               to achieve stride one access below */
+                            acc[ncontrib[i] + i * nima]
+                                = rsc[4 + 8*p] *
+                                (rsc[0 + 8*p] * pi[i - 1 - sizex] +
+                                 rsc[1 + 8*p] * pi[i     - sizex] +
+                                 rsc[2 + 8*p] * pi[i + 1 - sizex] +
+                                 rsc[3 + 8*p] * pi[i + 2 - sizex] )
+                                + rsc[5 + 8*p] *
+                                (rsc[0 + 8*p] * pi[i - 1] +
+                                 rsc[1 + 8*p] * pi[i    ] +
+                                 rsc[2 + 8*p] * pi[i + 1] +
+                                 rsc[3 + 8*p] * pi[i + 2] )
+                                + rsc[6 + 8*p] *
+                                (rsc[0 + 8*p] * pi[i - 1 + sizex] +
+                                 rsc[1 + 8*p] * pi[i     + sizex] +
+                                 rsc[2 + 8*p] * pi[i + 1 + sizex] +
+                                 rsc[3 + 8*p] * pi[i + 2 + sizex] )
+                                + rsc[7 + 8*p] *
+                                (rsc[0 + 8*p] * pi[i - 1 + sizex * 2] +
+                                 rsc[1 + 8*p] * pi[i     + sizex * 2] +
+                                 rsc[2 + 8*p] * pi[i + 1 + sizex * 2] +
+                                 rsc[3 + 8*p] * pi[i + 2 + sizex * 2]);
+
+                            ncontrib[i]++;
+                            myintp_count++;
+                        }
+                    }
                 }
             }
         }
@@ -358,8 +379,21 @@ static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
             if (ncontrib[i] > rtot) {
                 double finpix = 0.0;
                 double * acci = acc + i * nima;
+
+                /* Place rmin and rmax samples at the ends */
+#ifdef CPL_GEOM_USE_KTH
+                /* Equivalent, but slower in unit tests */
+                if (rmin > 0)
+                    (void)cpl_tools_get_kth_double(acci, ncontrib[i], rmin-1);
+                if (rmax > 0)
+                    (void)cpl_tools_get_kth_double(acci + rmin,
+                                                   ncontrib[i] - rmin,
+                                                   ncontrib[i] - rtot);
+#else
                 cpl_geom_img_get_min_max_double(acci, ncontrib[i],
                                                 rmin, rmax);
+#endif
+
                 for (p=rmin; p<(ncontrib[i]-rmax); p++) 
                     finpix += acci[p];
                 finpix /= (double)(ncontrib[i]-rtot);
@@ -370,9 +404,9 @@ static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
             } else {
                 cpl_image_reject(final, i+1, j+1);
             }
-            myflop_count += 35 * ncontrib[i];
             ncontrib[i] = 0;
         }
+        myflop_count += 35 * myintp_count;
     }
     cpl_tools_add_flops(myflop_count + 15 * nima);
 
@@ -382,6 +416,7 @@ static void CPL_TYPE_ADD(cpl_geom_img_offset_saa)(cpl_image * final,
     cpl_free(ncontrib);
     cpl_free((void*)ppi);
     cpl_free((void*)ppibpm);
+    cpl_free((void*)isint);
 
     return;
 }
@@ -411,16 +446,16 @@ void CPL_TYPE_ADD(cpl_geom_img_offset_saa_all)(cpl_image * final,
                                                const cpl_imagelist * ilist,
                                                const cpl_bivector * offs,
                                                const cpl_vector * xyprofile,
-                                               int tabsperpix)
+                                               cpl_size tabsperpix)
 
 {
 
-    const int          nima  = cpl_imagelist_get_size(ilist);
+    const cpl_size     nima  = cpl_imagelist_get_size(ilist);
     const cpl_image  * img1  = cpl_imagelist_get_const(ilist, 0);
-    const int          sizex = cpl_image_get_size_x(img1);
-    const int          sizey = cpl_image_get_size_y(img1);
-    const int          nx    = cpl_image_get_size_x(final);
-    const int          ny    = cpl_image_get_size_y(final);
+    const cpl_size     sizex = cpl_image_get_size_x(img1);
+    const cpl_size     sizey = cpl_image_get_size_y(img1);
+    const cpl_size     nx    = cpl_image_get_size_x(final);
+    const cpl_size     ny    = cpl_image_get_size_y(final);
 
     CPL_TYPE       * po = CPL_TYPE_ADD(cpl_image_get_data)(final);
     const double   * offs_x = cpl_bivector_get_x_data_const(offs);
@@ -439,7 +474,7 @@ void CPL_TYPE_ADD(cpl_geom_img_offset_saa_all)(cpl_image * final,
     cpl_flops        myintp_count = 0;
     cpl_flops        myaddp_count = 0;
     cpl_flops        myflop_count = nx * ny + 4 * nima;
-    int              i, j, p;
+    cpl_size         i, j, p;
 
     /* Triple loop */
     for (p = 0; p < nima; p++) {
@@ -449,11 +484,11 @@ void CPL_TYPE_ADD(cpl_geom_img_offset_saa_all)(cpl_image * final,
         const double      offset_y = start_y - offs_y[p];
         const double      suboff_x = offset_x - floor(offset_x);
         const double      suboff_y = offset_y - floor(offset_y);
-        const int         offset_i = (int)floor(offset_x);
-        const int         offset_j = (int)floor(offset_y);
+        const cpl_size    offset_i = (int)floor(offset_x);
+        const cpl_size    offset_j = (int)floor(offset_y);
         /* Which tabulated value index shall be used? */
-        const int         tabx = (int)(0.5 + suboff_x * (double)(tabsperpix));
-        const int         taby = (int)(0.5 + suboff_y * (double)(tabsperpix));
+        const cpl_size    tabx = (int)(0.5 + suboff_x * (double)(tabsperpix));
+        const cpl_size    taby = (int)(0.5 + suboff_y * (double)(tabsperpix));
 
         CPL_TYPE * poj = po;
         int      * pcj = pcontrib;
