@@ -1,4 +1,4 @@
-/* $Id: cpl_apertures-test.c,v 1.39 2011/12/20 11:06:30 llundin Exp $
+/* $Id: cpl_apertures-test.c,v 1.41 2012/04/04 12:05:51 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/20 11:06:30 $
- * $Revision: 1.39 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/04/04 12:05:51 $
+ * $Revision: 1.41 $
+ * $Name: cpl-6_1_1 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -194,8 +194,9 @@ int main(void)
 
     for (itype = 0; itype < sizeof(img_type)/sizeof(img_type[0]); itype++) {
         const cpl_size naperts = 2;
+        const cpl_type type = img_type[itype];
         cpl_image * labels = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_INT);
-        cpl_image * img = cpl_image_cast(imd, img_type[itype]);
+        cpl_image * img = cpl_image_cast(imd, type);
         cpl_image * imb = cpl_image_duplicate(img);
         cpl_mask  * bin;
         cpl_size    xpos, ypos;
@@ -205,7 +206,7 @@ int main(void)
 
         /* Create test image and the labels */
         cpl_msg_info(cpl_func, "Testing with a %d X %d %s-image",
-                     IMAGESZ, IMAGESZ, cpl_type_get_name(img_type[itype]));
+                     IMAGESZ, IMAGESZ, cpl_type_get_name(type));
 
         /* Test 3: with a 'bad' label image */
         nullapert = cpl_apertures_new_from_image(img, NULL);
@@ -233,6 +234,17 @@ int main(void)
 
         cpl_apertures_delete(aperts);
 
+        if (type != CPL_TYPE_INT) {
+            /* Test (error) handling of a complex image */
+            cpl_image * imgc = cpl_image_cast(imd, type | CPL_TYPE_COMPLEX);
+
+            aperts = cpl_apertures_new_from_image(imgc, labels);
+            cpl_test_error(CPL_ERROR_TYPE_MISMATCH);
+            cpl_test_null(aperts);
+            cpl_image_delete(imgc);
+        }
+
+        /* Test other error handling */
         error = cpl_image_set(labels, 1, 1, 2.0);
         cpl_test_eq_error(error, CPL_ERROR_NONE);
 
@@ -408,6 +420,22 @@ int main(void)
 
     cpl_apertures_delete(aperts);
 
+    /* Test error handling of cpl_apertures_extract_sigma() */
+    nullapert = cpl_apertures_extract_sigma(imd, 100.0);
+    cpl_test_error(CPL_ERROR_DATA_NOT_FOUND);
+    cpl_test_null(nullapert);
+
+    /* Test error handling of cpl_apertures_extract_mask() */
+    nullapert = cpl_apertures_extract_mask(imd, NULL);
+    cpl_test_error(CPL_ERROR_NULL_INPUT);
+    cpl_test_null(nullapert);
+
+    cpl_mask_delete(bpm);
+    bpm = cpl_mask_new(IMAGESZ2, IMAGESZ2);
+    nullapert = cpl_apertures_extract_mask(imd, bpm);
+    cpl_test_error(CPL_ERROR_DATA_NOT_FOUND);
+    cpl_test_null(nullapert);
+
     /* Test cpl_apertures_extract_window() */
     cpl_msg_info("","Test the apertures detection (threshold) on a zone");
     aperts = cpl_apertures_extract_window(imd, sigmas, IMAGESZ2/4, IMAGESZ2/4,
