@@ -1,4 +1,4 @@
-/* $Id: cpl_fit-test.c,v 1.44 2012/02/15 14:27:28 llundin Exp $
+/* $Id: cpl_fit-test.c,v 1.45 2012/05/21 14:47:00 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/15 14:27:28 $
- * $Revision: 1.44 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/21 14:47:00 $
+ * $Revision: 1.45 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1071,8 +1071,10 @@ static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
                                    &rms, &chisq, &covariance, &major, 
                                    &minor, &angle, &phys_cov);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
-    cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters, err_params, 
-                                 angle, major, minor, phys_cov));
+    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                     err_params, angle, major, minor,
+                                     phys_cov));
 
     cpl_matrix_delete(covariance);
     cpl_matrix_delete(phys_cov);
@@ -1100,8 +1102,10 @@ static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
                                    &minor, &angle, &phys_cov);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters, err_params, 
-                                 angle, major, minor, phys_cov));
+    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                     err_params, angle, major, minor,
+                                     phys_cov));
 
     cpl_matrix_delete(covariance);
     cpl_matrix_delete(phys_cov);
@@ -1127,8 +1131,10 @@ static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
                                    &minor, &angle, &phys_cov);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters, err_params,
-                                 angle, major, minor, phys_cov));
+    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                     err_params, angle, major, minor,
+                                     phys_cov));
 
     cpl_matrix_delete(covariance);
     cpl_matrix_delete(phys_cov);
@@ -1146,8 +1152,10 @@ static void cpl_fit_image_gaussian_tests(cpl_type pixeltype)
                                    &minor, &angle, NULL);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters, err_params,
-                                 angle, major, minor, NULL));
+    if (!error || cpl_msg_get_level() <= CPL_MSG_INFO)
+        cpl_test(check_gauss_success(p, u, e, fp, fu, fe, parameters,
+                                     err_params, angle, major, minor,
+                                     phys_cov));
 
     /*
      * Try different error situations:
