@@ -1,4 +1,4 @@
-/* $Id: cpl_gasgano.c,v 1.18 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_gasgano.c,v 1.21 2012/05/11 21:03:24 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.18 $
- * $Name: cpl-6_0 $
+ * $Date: 2012/05/11 21:03:24 $
+ * $Revision: 1.21 $
+ * $Name: cpl-6_1_1 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1220,7 +1220,7 @@ static cpl_error_code setCplMessaging(cpl_boolean startlog) {
 
         cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
 
-        return cpl_error_get_code();
+        return cpl_error_set_where(cpl_func);
     } else {
         return CPL_ERROR_NONE;
     }
